@@ -1,8 +1,8 @@
-/* $Id: bits.c 5498 2009-03-18 17:37:53Z esr $ */
+/* $Id: bits.c 6566 2009-11-20 03:51:06Z esr $ */
 /*
  * Bitfield extraction functions.  In each, start is a bit index (not
  * a byte index) and width is a bit width.  The width bounded above by
- * the bit width of a long long, which s 64 bits in all standard data
+ * the bit width of a long long, which is 64 bits in all standard data
  * models for 32- and 64-bit processors.
  *
  * The sbits() function assumes twos-complement arithmetic.
@@ -12,14 +12,13 @@
 #include "bits.h"
 #ifdef DEBUG
 #include <stdio.h>
-#include "gpsd_config.h"
 #include "gpsd.h"
 #endif /* DEBUG */
 
 #define BITS_PER_BYTE	8
 
 unsigned long long ubits(char buf[], unsigned int start, unsigned int width)
-/* extract a bitfield from the buffer as an unsigned big-endian long long */
+/* extract a (zero-origin) bitfield from the buffer as an unsigned big-endian long long */
 {
     unsigned long long fld = 0;
     unsigned int i;
