@@ -1,12 +1,12 @@
-/* $Id: bsd-base64.h 4794 2008-08-03 16:42:57Z ckuethe $ */
+/* $Id: bsd-base64.h 6908 2010-01-02 22:29:16Z esr $ */
 #ifndef _BSD_BASE64_H
 #define _BSD_BASE64_H
 
 #ifndef HAVE___B64_NTOP
 # ifndef HAVE_B64_NTOP
-int b64_ntop(u_char const *src, size_t srclength, char *target, 
+int b64_ntop(unsigned char const *src, size_t srclength, char *target, 
     size_t targsize);
-int b64_pton(char const *src, u_char *target, size_t targsize);
+int b64_pton(char const *src, unsigned char *target, size_t targsize);
 # endif /* !HAVE_B64_NTOP */
 # define __b64_ntop b64_ntop
 # define __b64_pton b64_pton
