@@ -1,4 +1,4 @@
-/* $Id: driver_evermore.c 5411 2009-03-07 15:35:13Z esr $ */
+/* $Id: driver_evermore.c 6566 2009-11-20 03:51:06Z esr $ */
 /*
  * This is the gpsd driver for EverMore GPSes operating in binary mode.
  * About the only thing this gives us that NMEA won't is TDOP.
@@ -118,11 +118,12 @@
 #include <string.h>
 #include <math.h>
 #include <ctype.h>
+#ifndef S_SPLINT_S
 #include <unistd.h>
+#endif /* S_SPLINT_S */
 #include <time.h>
 #include <stdio.h>
 
-#include "gpsd_config.h"
 #include "gpsd.h"
 #if defined(EVERMORE_ENABLE) && defined(BINARY_ENABLE)
 
@@ -137,7 +138,7 @@ gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size
 {
     unsigned char buf2[MAX_PACKET_LENGTH], *cp, *tp;
     size_t i, datalen;
-    unsigned int used, visible, satcnt;
+    unsigned int type, used, visible, satcnt;
     double version;
     gps_mask_t mask = 0;
 
@@ -155,31 +156,39 @@ gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size
 
     datalen -= 2;
 
+    /*@ -usedef @*/
     for (i = 0; i < (size_t)datalen; i++) {
 	*tp = *cp++;
 	if (*tp == 0x10) cp++;
 	tp++;
     }
 
+    type = (unsigned char)getub(buf2, 1);
+    /*@ +usedef @*/
+
     /*@ -usedef -compdef @*/
     gpsd_report(LOG_RAW, "EverMore packet type 0x%02x, length %zd: %s\n",
-	buf2[0], datalen, gpsd_hexdump_wrapper(buf2, datalen, LOG_RAW));
+	type, datalen, gpsd_hexdump_wrapper(buf2, datalen, LOG_RAW));
     /*@ +usedef +compdef @*/
 
     (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag),
-		   "EID%d",(int)buf2[0]);
+		   "EID%u", type);
 
-    switch (getub(buf2, 1))
+    session->cycle_end_reliable = true;
+
+    switch (type)
     {
     case 0x02:	/* Navigation Data Output */
-	session->gpsdata.fix.time = session->gpsdata.sentence_time
-	    = gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
+	/*@ ignore @*//*@ splint is confused @*/
+	session->gpsdata.fix.time =
+	    gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
+	/*@ end @*/
 	ecef_to_wgs84fix(&session->gpsdata, 
 			 getlesl(buf2, 8)*1.0, getlesl(buf2, 12)*1.0, getlesl(buf2, 16)*1.0,
 			 getlesw(buf2, 20)/10.0, getlesw(buf2, 22)/10.0, getlesw(buf2, 24)/10.0);
-	used = getub(buf2, 26) & 0x0f;
-	visible = (getub(buf2, 26) & 0xf0) >> 4;
-	version = getleuw(buf2, 27)/100.0;
+	used = (unsigned char)getub(buf2, 26) & 0x0f;
+	//visible = (getub(buf2, 26) & 0xf0) >> 4;
+	version = (uint)getleuw(buf2, 27)/100.0;
 	/* that's all the information in this packet */
 	if (used < 3)
 	    session->gpsdata.fix.mode = MODE_NO_FIX;
@@ -189,28 +198,36 @@ gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size
 	    session->gpsdata.fix.mode = MODE_3D;
 	    mask |= ALTITUDE_SET | CLIMB_SET;
 	}
-	gpsd_report(LOG_PROG, "NDO 0x02: version %3.2f, mode=%d, status=%d, visible=%d, used=%d\n",
-		    version,
-		    session->gpsdata.fix.mode,
-		    session->gpsdata.status,
-		    visible,
-		    used);
-	mask |= TIME_SET | LATLON_SET | TRACK_SET | SPEED_SET | MODE_SET | CYCLE_START_SET;
+	mask |= TIME_SET | LATLON_SET | TRACK_SET | SPEED_SET | MODE_SET;
 	if (session->subtype[0] == '\0') {
 	    (void)snprintf(session->subtype, sizeof(session->subtype), 
 		       "%3.2f", version);
 	    mask |= DEVICEID_SET;
 	}
-	return mask;
+	gpsd_report(LOG_DATA, "NDO 0x02: time=%.2f, lat=%.2f lon=%.2f alt=%.2f speed=%.2f track=%.2f climb=%.2f mode=%d subtype='%s' mask=%s\n",
+		    session->gpsdata.fix.time,
+		    session->gpsdata.fix.latitude,
+		    session->gpsdata.fix.longitude,
+		    session->gpsdata.fix.altitude,
+		    session->gpsdata.fix.speed,
+		    session->gpsdata.fix.track,
+		    session->gpsdata.fix.climb,
+		    session->gpsdata.fix.mode,
+		    session->gpsdata.dev.subtype,
+		    gpsd_maskdump(mask));
+	return mask | CLEAR_SET | REPORT_SET;
 
     case 0x04:	/* DOP Data Output */
-	session->gpsdata.fix.time = session->gpsdata.sentence_time
-	    = gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
-	session->gpsdata.gdop = (double)getub(buf2, 8)*0.1;
-	session->gpsdata.pdop = (double)getub(buf2, 9)*0.1;
-	session->gpsdata.hdop = (double)getub(buf2, 10)*0.1;
-	session->gpsdata.vdop = (double)getub(buf2, 11)*0.1;
-	session->gpsdata.tdop = (double)getub(buf2, 12)*0.1;
+	/*@ ignore @*//*@ splint is confused @*/
+	session->gpsdata.fix.time =
+	    gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
+	/*@ end @*/
+	clear_dop(&session->gpsdata.dop);
+	session->gpsdata.dop.gdop = (double)getub(buf2, 8)*0.1;
+	session->gpsdata.dop.pdop = (double)getub(buf2, 9)*0.1;
+	session->gpsdata.dop.hdop = (double)getub(buf2, 10)*0.1;
+	session->gpsdata.dop.vdop = (double)getub(buf2, 11)*0.1;
+	session->gpsdata.dop.tdop = (double)getub(buf2, 12)*0.1;
 	switch (getub(buf2, 13)) {
 	case 0:	/* no position fix */
 	case 1:	/* manual calls this "1D navigation" */
@@ -231,32 +248,41 @@ gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size
 	    break;
 	}
 	/* that's all the information in this packet */
-	gpsd_report(LOG_PROG, "DDO 0x04: mode=%d, status=%d\n", 
+	mask = TIME_SET | DOP_SET | MODE_SET | STATUS_SET;
+	gpsd_report(LOG_DATA, "DDO 0x04: gdop=%.2f pdop=%.2f hdop=%.2f vdop=%.2f tdop=%.2f mode=%d, status=%d mask={TIME| DOP|MODE|STATUS}\n", 
+		    session->gpsdata.dop.gdop,
+		    session->gpsdata.dop.pdop,
+		    session->gpsdata.dop.hdop,
+		    session->gpsdata.dop.vdop,
+		    session->gpsdata.dop.tdop,
 		    session->gpsdata.fix.mode,
 		    session->gpsdata.status);
-	return TIME_SET | DOP_SET | MODE_SET | STATUS_SET;
+	return mask;
 
     case 0x06:	/* Channel Status Output */
-	session->gpsdata.fix.time = session->gpsdata.sentence_time
-	    = gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
-	session->gpsdata.satellites = (int)getub(buf2, 8);
+	/*@ ignore @*//*@ splint is confused @*/
+	session->gpsdata.skyview_time =
+	    gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
+	/*@ end @*/
+	session->gpsdata.satellites_visible = (int)getub(buf2, 8);
 	session->gpsdata.satellites_used = 0;
 	memset(session->gpsdata.used, 0, sizeof(session->gpsdata.used));
-	if (session->gpsdata.satellites > 12) {
+	if (session->gpsdata.satellites_visible > 12) {
 		gpsd_report(LOG_WARN, "Warning: EverMore packet has information about %d satellites!\n",
-				session->gpsdata.satellites);
+				session->gpsdata.satellites_visible);
 	}
-	if (session->gpsdata.satellites > EVERMORE_CHANNELS) session->gpsdata.satellites = EVERMORE_CHANNELS;
+	if (session->gpsdata.satellites_visible > EVERMORE_CHANNELS) 
+	    session->gpsdata.satellites_visible = EVERMORE_CHANNELS;
 	satcnt = 0;
-	for (i = 0; i < (size_t)session->gpsdata.satellites; i++) {
+	for (i = 0; i < (size_t)session->gpsdata.satellites_visible; i++) {
 	    int prn;
 	    // channel = getub(buf2, 7*i+7+2)
-            prn = (int)getub(buf2, 7*i+7+3);
+	    prn = (int)getub(buf2, 7*i+7+3);
 	    if (prn == 0) continue;  /* satellite record is not valid */
 	    session->gpsdata.PRN[satcnt] = prn;
 	    session->gpsdata.azimuth[satcnt] = (int)getleuw(buf2, 7*i+7+4);
 	    session->gpsdata.elevation[satcnt] = (int)getub(buf2, 7*i+7+6);
-	    session->gpsdata.ss[satcnt] = (int)getub(buf2, 7*i+7+7);
+	    session->gpsdata.ss[satcnt] = (float)getub(buf2, 7*i+7+7);
 	    /*
 	     * Status bits at offset 8:
 	     * bit0 = 1 satellite acquired
@@ -272,24 +298,29 @@ gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size
 	    }
 
 	    satcnt++;
-		
 	}
-	session->gpsdata.satellites = (int)satcnt;
+	session->gpsdata.satellites_visible = (int)satcnt;
 	/* that's all the information in this packet */
-	gpsd_report(LOG_PROG, "CSO 0x04: %d satellites used\n", 
-		    session->gpsdata.satellites_used);
-	return TIME_SET | SATELLITE_SET | USED_SET;
+	mask = SATELLITE_SET | USED_SET;
+	gpsd_report(LOG_DATA, "CSO 0x06: time=%.2f used=%d visible=%d mask={TIME|SATELLITE|USED}\n",
+		    session->gpsdata.fix.time,
+		    session->gpsdata.satellites_used,
+		    session->gpsdata.satellites_visible);
+	return mask;
 
     case 0x08:	/* Measurement Data Output */
 	/* clock offset is a manufacturer diagnostic */
 	/* (int)getleuw(buf2, 8);  clock offset, 29000..29850 ?? */
-	session->gpsdata.fix.time = session->gpsdata.sentence_time
-	    = gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
-	visible = getub(buf2, 10);
+	/*@ ignore @*//*@ splint is confused @*/
+	session->gpsdata.fix.time = 
+	    gpstime_to_unix((int)getleuw(buf2, 2), getleul(buf2, 4)*0.01) - session->context->leap_seconds;
+	/*@ end @*/
+	visible = (unsigned char)getub(buf2, 10);
 	/* FIXME: read full statellite status for each channel */
 	/* we can get pseudo range (m), delta-range (m/s), doppler (Hz) and status for each channel */
 	/* gpsd_report(LOG_PROG, "MDO 0x04: visible=%d\n", visible); */
-	gpsd_report(LOG_PROG, "MDO 0x04:\n");
+	gpsd_report(LOG_DATA, "MDO 0x04: time=%.2f mask={TIME}\n",
+		    session->gpsdata.fix.time);
 	return TIME_SET;
 
     case 0x20:	/* LogConfig Info, could be used as a probe for EverMore GPS */
@@ -308,7 +339,7 @@ gps_mask_t evermore_parse(struct gps_device_t *session, unsigned char *buf, size
 
     default:
 	gpsd_report(LOG_WARN,
-	    "unknown EverMore packet id 0x%02x, length %zd: %s\n", buf2[0],
+	    "unknown EverMore packet EID 0x%02x, length %zd: %s\n", buf2[0],
 	    datalen, gpsd_hexdump_wrapper(buf2, datalen, LOG_IO));
 	return 0;
     }
@@ -319,7 +350,7 @@ static gps_mask_t evermore_parse_input(struct gps_device_t *session)
 {
     gps_mask_t st;
 
-    if (session->packet.type == EVERMORE_PACKET){
+    if (session->packet.type == EVERMORE_PACKET) {
 	st = evermore_parse(session, session->packet.outbuffer, session->packet.outbuflen);
 	return st;
 #ifdef NMEA_ENABLE
@@ -381,9 +412,9 @@ static bool evermore_speed(struct gps_device_t *session,
 			   speed_t speed, char parity, int stopbits)
 {
     /*@ -type @*/
-    gpsd_report(LOG_PROG, "evermore_speed(%d%c%d)\n", speed, parity, stopbits);
+    gpsd_report(LOG_PROG, "evermore_speed(%u%c%d)\n", speed, parity, stopbits);
     /* parity and stopbit switching aren't available on this chip */
-    if (parity!=(int)session->gpsdata.parity || stopbits!=(int)session->gpsdata.parity) {
+    if (parity!=session->gpsdata.dev.parity || stopbits!=(int)session->gpsdata.dev.parity) {
 	return false;
     } else {
 	unsigned char tmp8;
@@ -458,26 +489,30 @@ static void evermore_mode(struct gps_device_t *session, int mode)
     if (mode == MODE_NMEA) {
         /* NMEA */
         (void) evermore_protocol(session, 1);
-        session->gpsdata.driver_mode = MODE_NMEA;
+        session->gpsdata.dev.driver_mode = MODE_NMEA;
         (void) evermore_nmea_config(session, 1); /* configure NMEA messages for gpsd */
     } else {
         /* binary */
         (void) evermore_protocol(session, 0);
         session->back_to_nmea = false;
-        session->gpsdata.driver_mode = MODE_BINARY;
+        session->gpsdata.dev.driver_mode = MODE_BINARY;
     }
 }
 
-static void evermore_configurator(struct gps_device_t *session, unsigned int seq)
+static void evermore_event_hook(struct gps_device_t *session, event_t event)
 {
-    gpsd_report(LOG_PROG, "evermore_configurator(%d)\n", seq);
-    (void) evermore_nmea_config(session, 1); /* configure NMEA messages for gpsd (GPGSV every 5s) */
-    if (seq == 0) {
+    /*
+     * FIXME: It might not be necessary to call this on reactivate.
+     * Experiment to see if the holds its settings through a close.
+     */
+    if (event == event_identified || event == event_reactivate) {
         if (session->packet.type == NMEA_PACKET) {
-	    gpsd_report(LOG_WARN, "NMEA_PACKET packet\n");
-        }
+	    (void) evermore_nmea_config(session, 1); /* configure NMEA messages for gpsd (GPGSV every 5s) */
+        } 
         (void) evermore_mode(session, 1); /* switch GPS to binary mode */
         session->back_to_nmea = true;
+    } else if (event == event_deactivate) {
+	(void) evermore_nmea_config(session, 0); /* configure NMEA messages to default */
     }
 }
 
@@ -502,12 +537,6 @@ static bool evermore_rate_switcher(struct gps_device_t *session, double rate)
     }
     /*@ -charint @*/
 }
-
-static void evermore_revert(struct gps_device_t *session)
-{
-    gpsd_report(LOG_PROG, "evermore_revert\n");
-    (void) evermore_nmea_config(session, 0); /* configure NMEA messages to default */
-}
 #endif /* ALLOW_RECONFIGURE */
 
 
@@ -518,23 +547,19 @@ const struct gps_type_t evermore_binary =
     .packet_type    = EVERMORE_PACKET,		/* lexer packet type */
     .trigger        = "$PEMT,", 		/* recognize the type */
     .channels       = EVERMORE_CHANNELS,	/* consumer-grade GPS */
-    .probe_wakeup   = NULL,			/* no wakeup to be done before hunt */
     .probe_detect   = NULL,			/* no probe */
-    .probe_subtype  = NULL,			/* no subtype probing */
     .get_packet     = generic_get,		/* use generic one */
     .parse_packet   = evermore_parse_input,	/* parse message packets */
     .rtcm_writer    = pass_rtcm,		/* send RTCM data straight */
-#ifdef ALLOW_CONTROLSEND
-    .control_send   = evermore_control_send,	/* how to send a control string */
-#endif /* ALLOW_CONTROLSEND */
+    .event_hook     = evermore_event_hook,	/* lifetime event handler */
 #ifdef ALLOW_RECONFIGURE
-    .configurator   = evermore_configurator,	/* switch to binary */
     .speed_switcher = evermore_speed,		/* we can change baud rates */
     .mode_switcher  = evermore_mode,		/* there is a mode switcher */
     .rate_switcher  = evermore_rate_switcher,	/* change sample rate */
     .min_cycle      = 1,			/* ignore, no rate switch */
-    .revert         = evermore_revert,		/* reversion code */
 #endif /* ALLOW_RECONFIGURE */
-    .wrapup         = NULL,			/* wrapup method */
+#ifdef ALLOW_CONTROLSEND
+    .control_send   = evermore_control_send,	/* how to send a control string */
+#endif /* ALLOW_CONTROLSEND */
 };
 #endif /* defined(EVERMORE_ENABLE) && defined(BINARY_ENABLE) */
