@@ -1,4 +1,4 @@
-/* $Id: driver_italk.c 5410 2009-03-07 15:32:23Z esr $ */
+/* $Id: driver_italk.c 7003 2010-02-16 22:34:15Z esr $ */
 /*
  * Driver for the iTalk binary protocol used by FasTrax
  */
@@ -8,11 +8,12 @@
 #include <string.h>
 #include <math.h>
 #include <ctype.h>
+#ifndef S_SPLINT_S
 #include <unistd.h>
+#endif /* S_SPLINT_S */
 #include <time.h>
 #include <stdio.h>
 
-#include "gpsd_config.h"
 #include "gpsd.h"
 #if defined(ITRAX_ENABLE) && defined(BINARY_ENABLE)
 
@@ -23,37 +24,37 @@ static gps_mask_t italk_parse(struct gps_device_t *, unsigned char *, size_t);
 static gps_mask_t decode_itk_navfix(struct gps_device_t *, unsigned char *, size_t);
 static gps_mask_t decode_itk_prnstatus(struct gps_device_t *, unsigned char *, size_t);
 static gps_mask_t decode_itk_utcionomodel(struct gps_device_t *, unsigned char *, size_t);
+static gps_mask_t decode_itk_subframe(struct gps_device_t *, unsigned char *, size_t);
 
 static gps_mask_t decode_itk_navfix(struct gps_device_t *session, unsigned char *buf, size_t len)
 {
     unsigned int tow;
     unsigned short gps_week, flags, cflags, pflags;
     gps_mask_t mask = 0;
-    double epx, epy, epz, evx, evy, evz;
+    double epx, epy, epz, evx, evy, evz, eph;
     double t;
 
-    if (len != 296){
+    if (len != 296) {
 	gpsd_report(LOG_PROG, "ITALK: bad NAV_FIX (len %zu, should be 296)\n",
 		    len);
 	return -1;
     }
 
-    flags = getleuw(buf, 7 + 4);
-    cflags = getleuw(buf, 7 + 6);
-    pflags = getleuw(buf, 7 + 8);
+    flags = (ushort)getleuw(buf, 7 + 4);
+    cflags = (ushort)getleuw(buf, 7 + 6);
+    pflags = (ushort)getleuw(buf, 7 + 8);
 
     session->gpsdata.status = STATUS_NO_FIX;
     session->gpsdata.fix.mode = MODE_NO_FIX;
-    mask =  ONLINE_SET | MODE_SET | STATUS_SET | CYCLE_START_SET;
+    mask =  ONLINE_SET | MODE_SET | STATUS_SET | CLEAR_SET;
 
     /* just bail out if this fix is not marked valid */
     if (0 != (pflags & FIX_FLAG_MASK_INVALID) || 0 == (flags & FIXINFO_FLAG_VALID))
 	return mask;
 
     gps_week = (ushort)getlesw(buf, 7 + 82);
-    tow = getleul(buf, 7 + 84);
+    tow = (uint)getleul(buf, 7 + 84);
     t = gpstime_to_unix((int)gps_week, tow/1000.0) - session->context->leap_seconds;
-    session->gpsdata.sentence_time = t;
     session->gpsdata.fix.time = t;
     mask |= TIME_SET;
 
@@ -65,7 +66,9 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session, unsigned char
     evz = (double)(getlesl(buf, 7 + 194)/1000.0);
     ecef_to_wgs84fix(&session->gpsdata, epx, epy, epz, evx, evy, evz);
     mask |= LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET  ;
-    session->gpsdata.fix.eph = (double)(getlesl(buf, 7 + 252)/100.0);
+    eph = (double)(getlesl(buf, 7 + 252)/100.0);
+    /* eph is a circular error, sqrt(epx**2 + epy**2) */
+    session->gpsdata.fix.epx = session->gpsdata.fix.epy = eph/sqrt(2);
     session->gpsdata.fix.eps = (double)(getlesl(buf, 7 + 254)/100.0);
 
     #define MAX(a,b) (((a) > (b)) ? (a) : (b))
@@ -73,16 +76,17 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session, unsigned char
 	(int)MAX(getleuw(buf, 7 + 12), getleuw(buf, 7 + 14));
     mask |= USED_SET ;
 
-    if (flags & FIX_CONV_DOP_VALID){
-	session->gpsdata.hdop = (double)(getleuw(buf, 7 + 56)/100.0);
-	session->gpsdata.gdop = (double)(getleuw(buf, 7 + 58)/100.0);
-	session->gpsdata.pdop = (double)(getleuw(buf, 7 + 60)/100.0);
-	session->gpsdata.vdop = (double)(getleuw(buf, 7 + 62)/100.0);
-	session->gpsdata.tdop = (double)(getleuw(buf, 7 + 64)/100.0);
-	mask |= HDOP_SET | GDOP_SET | PDOP_SET | VDOP_SET | TDOP_SET;
+    if (flags & FIX_CONV_DOP_VALID) {
+	clear_dop(&session->gpsdata.dop);
+	session->gpsdata.dop.hdop = (double)(getleuw(buf, 7 + 56)/100.0);
+	session->gpsdata.dop.gdop = (double)(getleuw(buf, 7 + 58)/100.0);
+	session->gpsdata.dop.pdop = (double)(getleuw(buf, 7 + 60)/100.0);
+	session->gpsdata.dop.vdop = (double)(getleuw(buf, 7 + 62)/100.0);
+	session->gpsdata.dop.tdop = (double)(getleuw(buf, 7 + 64)/100.0);
+	mask |= DOP_SET;
     }
 
-    if ((pflags & FIX_FLAG_MASK_INVALID) == 0 && (flags & FIXINFO_FLAG_VALID) != 0){
+    if ((pflags & FIX_FLAG_MASK_INVALID) == 0 && (flags & FIXINFO_FLAG_VALID) != 0) {
 	if (pflags & FIX_FLAG_3DFIX)
 	    session->gpsdata.fix.mode = MODE_3D;
 	else
@@ -94,6 +98,22 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session, unsigned char
 	    session->gpsdata.status = STATUS_FIX;
     }
 
+    gpsd_report(LOG_DATA, "NAV_FIX: time=%.2f, lat=%.2f lon=%.2f alt=%.f speed=%.2f track=%.2f climb=%.2f mode=%d status=%d gdop=%.2f pdop=%.2f hdop=%.2f vdop=%.2f tdop=%.2f mask=%s\n",
+		session->gpsdata.fix.time,
+		session->gpsdata.fix.latitude,
+		session->gpsdata.fix.longitude,
+		session->gpsdata.fix.altitude,
+		session->gpsdata.fix.speed,
+		session->gpsdata.fix.track,
+		session->gpsdata.fix.climb,
+		session->gpsdata.fix.mode,
+		session->gpsdata.status,
+		session->gpsdata.dop.gdop,
+		session->gpsdata.dop.pdop,
+		session->gpsdata.dop.hdop,
+		session->gpsdata.dop.vdop,
+		session->gpsdata.dop.tdop,
+		gpsd_maskdump(mask));
     return mask;
 }
 
@@ -102,41 +122,51 @@ static gps_mask_t decode_itk_prnstatus(struct gps_device_t *session, unsigned ch
     unsigned int i, tow, nsv, nchan, st;
     unsigned short gps_week;
     double t;
+    gps_mask_t mask;
 
-    if (len < 62){
+    if (len < 62) {
 	gpsd_report(LOG_PROG, "ITALK: runt PRN_STATUS (len=%zu)\n", len);
-	return ERROR_SET;
+	mask = ERROR_SET;
     }
+    else
+    {
+	gps_week = (ushort)getleuw(buf, 7 + 4);
+	tow = (uint)getleul(buf, 7 + 6);
+	t = gpstime_to_unix((int)gps_week, tow/1000.0) - session->context->leap_seconds;
+	session->gpsdata.skyview_time = t;
 
-    gps_week = getleuw(buf, 7 + 4);
-    tow = getleul(buf, 7 + 6);
-    t = gpstime_to_unix((int)gps_week, tow/1000.0) - session->context->leap_seconds;
-    session->gpsdata.sentence_time = session->gpsdata.fix.time = t;
-
-    gpsd_zero_satellites(&session->gpsdata);
-    nsv = 0;
-    nchan = (unsigned int)getleuw(buf, 7 +50);
-    if (nchan > MAX_NR_VISIBLE_PRNS)
+	gpsd_zero_satellites(&session->gpsdata);
+	nsv = 0;
+	nchan = (unsigned int)getleuw(buf, 7 +50);
+	if (nchan > MAX_NR_VISIBLE_PRNS)
 	    nchan = MAX_NR_VISIBLE_PRNS;
-    for (i = st = 0; i < nchan; i++) {
-	unsigned int off = 7+ 52 + 10 * i;
-	unsigned short flags;
+	for (i = st = 0; i < nchan; i++) {
+	    unsigned int off = 7+ 52 + 10 * i;
+	    unsigned short flags;
 
-	flags = getleuw(buf, off);
-	session->gpsdata.ss[i]		= (int)getleuw(buf, off+2)&0xff;
-	session->gpsdata.PRN[i]		= (int)getleuw(buf, off+4)&0xff;
-	session->gpsdata.elevation[i]	= (int)getlesw(buf, off+6)&0xff;
-	session->gpsdata.azimuth[i]	= (int)getlesw(buf, off+8)&0xff;
-	if (session->gpsdata.PRN[i]){
-	    st++;
-	    if (flags & PRN_FLAG_USE_IN_NAV)
-		session->gpsdata.used[nsv++] = session->gpsdata.PRN[i];
+	    flags = (ushort)getleuw(buf, off);
+	    session->gpsdata.ss[i]		= (float)(getleuw(buf, off+2)&0xff);
+	    session->gpsdata.PRN[i]		= (int)getleuw(buf, off+4)&0xff;
+	    session->gpsdata.elevation[i]	= (int)getlesw(buf, off+6)&0xff;
+	    session->gpsdata.azimuth[i]	= (int)getlesw(buf, off+8)&0xff;
+	    if (session->gpsdata.PRN[i]) {
+		st++;
+		if (flags & PRN_FLAG_USE_IN_NAV)
+		    session->gpsdata.used[nsv++] = session->gpsdata.PRN[i];
+	    }
 	}
-    }
-    session->gpsdata.satellites = (int)st;
-    session->gpsdata.satellites_used = (int)nsv;
+	session->gpsdata.satellites_visible = (int)st;
+	session->gpsdata.satellites_used = (int)nsv;
+	mask = USED_SET | SATELLITE_SET;;
 
-    return USED_SET | SATELLITE_SET | TIME_SET;
+	gpsd_report(LOG_DATA,
+		    "PRN_STATUS: time=%.2f visible=%d used=%d mask={USED|SATELLITE}\n",
+		    session->gpsdata.fix.time,
+		    session->gpsdata.satellites_visible,
+		    session->gpsdata.satellites_used);
+    }
+
+    return mask;
 }
 
 static gps_mask_t decode_itk_utcionomodel(struct gps_device_t *session, unsigned char *buf, size_t len)
@@ -146,13 +176,13 @@ static gps_mask_t decode_itk_utcionomodel(struct gps_device_t *session, unsigned
     unsigned short gps_week, flags;
     double t;
 
-    if (len != 64){
+    if (len != 64) {
 	gpsd_report(LOG_PROG,
 		    "ITALK: bad UTC_IONO_MODEL (len %zu, should be 64)\n", len);
 	return ERROR_SET;
     }
 
-    flags = getleuw(buf, 7);
+    flags = (ushort)getleuw(buf, 7);
     if (0 == (flags & UTC_IONO_MODEL_UTCVALID))
 	return 0;
 
@@ -160,14 +190,59 @@ static gps_mask_t decode_itk_utcionomodel(struct gps_device_t *session, unsigned
     if (session->context->leap_seconds < leap)
 	session->context->leap_seconds = leap;
 
-    gps_week = getleuw(buf, 7 + 36);
-    tow = getleul(buf, 7 + 38);
+    gps_week = (ushort)getleuw(buf, 7 + 36);
+    tow = (uint)getleul(buf, 7 + 38);
     t = gpstime_to_unix((int)gps_week, tow/1000.0) - session->context->leap_seconds;
-    session->gpsdata.sentence_time = session->gpsdata.fix.time = t;
+    session->gpsdata.fix.time = t;
 
+    gpsd_report(LOG_DATA,
+		"UTC_IONO_MODEL: time=%.2f mask={TIME}\n",
+		session->gpsdata.fix.time);
     return TIME_SET;
 }
 
+static gps_mask_t decode_itk_subframe(struct gps_device_t *session, unsigned char *buf, size_t len)
+{
+    unsigned short flags, prn, sf;
+    unsigned int words[10];
+
+    if (len != 64) {
+	gpsd_report(LOG_PROG,
+		    "ITALK: bad SUBFRAME (len %zu, should be 64)\n", len);
+	return ERROR_SET;
+    }
+
+    flags = (ushort)getleuw(buf, 7 + 4);
+    prn = (ushort)getleuw(buf, 7 + 6);
+    sf = (ushort)getleuw(buf, 7 + 8);
+    gpsd_report(LOG_PROG, "iTalk SUBFRAME prn %u sf %u - decode %s %s\n",
+		prn, sf,
+		flags & SUBFRAME_WORD_FLAG_MASK ? "error" : "ok",
+		flags & SUBFRAME_GPS_PREAMBLE_INVERTED ? "(inverted)" : "");
+    if (flags & SUBFRAME_WORD_FLAG_MASK)
+	return ONLINE_SET | ERROR_SET; // don't try decode an erroneous packet
+
+    /*
+     * Timo says "SUBRAME message contains decoded navigation message subframe
+     * words with parity checking done but parity bits still present."
+     */
+    /*@-type@*/
+    words[0] = (getbeul(buf, 7 + 14) & 0x3fffffff) >> 6;
+    words[1] = (getleul(buf, 7 + 18) & 0x3fffffff) >> 6;
+    words[2] = (getleul(buf, 7 + 22) & 0x3fffffff) >> 6;
+    words[3] = (getleul(buf, 7 + 26) & 0x3fffffff) >> 6;
+    words[4] = (getleul(buf, 7 + 30) & 0x3fffffff) >> 6;
+    words[5] = (getleul(buf, 7 + 34) & 0x3fffffff) >> 6;
+    words[6] = (getleul(buf, 7 + 38) & 0x3fffffff) >> 6;
+    words[7] = (getleul(buf, 7 + 42) & 0x3fffffff) >> 6;
+    words[8] = (getleul(buf, 7 + 46) & 0x3fffffff) >> 6;
+    words[9] = (getleul(buf, 7 + 50) & 0x3fffffff) >> 6;
+    /*@+type@*/
+
+    gpsd_interpret_subframe(session, words);
+    return ONLINE_SET;
+}
+
 /*@ +charint @*/
 static gps_mask_t italk_parse(struct gps_device_t *session, unsigned char *buf, size_t len)
 {
@@ -182,11 +257,13 @@ static gps_mask_t italk_parse(struct gps_device_t *session, unsigned char *buf,
     gpsd_report(LOG_RAW, "raw italk packet type 0x%02x length %zu: %s\n",
 	type, len, gpsd_hexdump_wrapper(buf, len, LOG_RAW));
 
+    session->cycle_end_reliable = true;
+
     switch (type)
     {
     case ITALK_NAV_FIX:
 	gpsd_report(LOG_IO, "iTalk NAV_FIX len %zu\n", len);
-	mask = decode_itk_navfix(session, buf, len);
+	mask = decode_itk_navfix(session, buf, len) | (CLEAR_SET|REPORT_SET);
 	break;
     case ITALK_PRN_STATUS:
 	gpsd_report(LOG_IO, "iTalk PRN_STATUS len %zu\n", len);
@@ -213,7 +290,7 @@ static gps_mask_t italk_parse(struct gps_device_t *session, unsigned char *buf,
 	gpsd_report(LOG_IO, "iTalk RAW_EPHEMERIS len %zu\n", len);
 	break;
     case ITALK_SUBFRAME:
-	gpsd_report(LOG_IO, "iTalk SUBFRAME len %zu\n", len);
+	mask = decode_itk_subframe(session, buf, len);
 	break;
     case ITALK_BIT_STREAM:
 	gpsd_report(LOG_IO, "iTalk BIT_STREAM len %zu\n", len);
@@ -272,14 +349,14 @@ static gps_mask_t italk_parse_input(struct gps_device_t *session)
 {
     gps_mask_t st;
 
-    if (session->packet.type == ITALK_PACKET){
+    if (session->packet.type == ITALK_PACKET) {
 	st = italk_parse(session, session->packet.outbuffer, session->packet.outbuflen);
-	session->gpsdata.driver_mode = MODE_BINARY;	/* binary */
+	session->gpsdata.dev.driver_mode = MODE_BINARY;	/* binary */
 	return st;
 #ifdef NMEA_ENABLE
     } else if (session->packet.type == NMEA_PACKET) {
 	st = nmea_parse((char *)session->packet.outbuffer, session);
-	session->gpsdata.driver_mode = MODE_NMEA;	/* NMEA */
+	session->gpsdata.dev.driver_mode = MODE_NMEA;	/* NMEA */
 	return st;
 #endif /* NMEA_ENABLE */
     } else
@@ -336,20 +413,24 @@ static void italk_mode(struct gps_device_t *session,  int mode)
 {
     if (mode == MODE_NMEA) {
 	(void)italk_set_mode(session, 
-			     session->gpsdata.baudrate,
-			     (char)session->gpsdata.parity,
-			     (int)session->gpsdata.stopbits,
+			     session->gpsdata.dev.baudrate,
+			     (char)session->gpsdata.dev.parity,
+			     (int)session->gpsdata.dev.stopbits,
 			     false);
     }
 }
 
-static void italk_configurator(struct gps_device_t *session, unsigned int seq)
+static void italk_event_hook(struct gps_device_t *session, event_t event)
 {
-    if (seq == 0 && session->packet.type == NMEA_PACKET)
+    /*
+     * FIXME: It might not be necessary to call this on reactivate.
+     * Experiment to see if the holds its settings through a close.
+     */
+    if ((event == event_identified || event == event_reactivate) && session->packet.type == NMEA_PACKET)
 	(void)italk_set_mode(session, 
-			     session->gpsdata.baudrate, 
-			     (char)session->gpsdata.parity,
-			     (int)session->gpsdata.stopbits,
+			     session->gpsdata.dev.baudrate, 
+			     (char)session->gpsdata.dev.parity,
+			     (int)session->gpsdata.dev.stopbits,
 			     true);
 }
 #endif /* ALLOW_RECONFIGURE */
@@ -370,24 +451,20 @@ const struct gps_type_t italk_binary =
     .packet_type    = ITALK_PACKET,	/* associated lexer packet type */
     .trigger	    = NULL,		/* recognize the type */
     .channels       = 12,		/* consumer-grade GPS */
-    .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
     .probe_detect   = NULL,		/* how to detect at startup time */
-    .probe_subtype  = NULL,		/* initialize the device */
     .get_packet     = generic_get,	/* use generic packet grabber */
     .parse_packet   = italk_parse_input,/* parse message packets */
     .rtcm_writer    = pass_rtcm,	/* send RTCM data straight */
-#ifdef ALLOW_CONTROLSEND
-    .control_send   = italk_control_send,	/* how to send a control string */
-#endif /* ALLOW_CONTROLSEND */
+    .event_hook     = italk_event_hook,	/* lifetime event handler */
 #ifdef ALLOW_RECONFIGURE
-    .configurator   = italk_configurator,/* configure the device */
     .speed_switcher = italk_speed,	/* we can change baud rates */
     .mode_switcher  = italk_mode,	/* there is a mode switcher */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-    .revert         = NULL,		/* no setting-reversion method */
 #endif /* ALLOW_RECONFIGURE */
-    .wrapup         = NULL,		/* no close hook */
+#ifdef ALLOW_CONTROLSEND
+    .control_send   = italk_control_send,	/* how to send a control string */
+#endif /* ALLOW_CONTROLSEND */
 };
 #endif /* defined(ITRAX_ENABLE) && defined(BINARY_ENABLE) */
 
@@ -423,7 +500,6 @@ const struct gps_type_t italk_binary =
 static int literal_send(int fd, const char *fmt, ... )
 /* ship a raw command to the GPS */
 {
-    ssize_t status;
     va_list ap;
 
     va_start(ap, fmt) ;
@@ -456,10 +532,10 @@ static void itrax_probe_subtype(struct gps_device_t *session, unsigned int seq)
 }
 
 #ifdef ALLOW_RECONFIGURE
-static void itrax_configurator(struct gps_device_t *session, int seq)
+static void itrax_configurator(struct gps_device_t event_t event, *session)
 /* set synchronous mode */
 {
-    if (seq == 0) {
+    if (event == event_configure && session.packet_counter == 0) {
 	(void)literal_send(session->gpsdata.gps_fd, "$PFST,SYNCMODE,1\r\n");
 	(void)literal_send(session->gpsdata.gps_fd,
 		    ITRAX_MODESTRING, session->gpsdata.baudrate);
@@ -496,24 +572,20 @@ const static struct gps_type_t itrax = {
     .packet_type    = NMEA_PACKET;	/* associated lexer packet type */
     .trigger        = "$PFST,OK",	/* tells us to switch to Itrax */
     .channels       = 12,		/* consumer-grade GPS */
-    .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
     .probe_detect   = NULL,		/* no probe */
-    .probe_subtype  = itrax_probe_subtype,	/* initialize */
     .get_packet     = generic_get,	/* how to get a packet */
     .parse_packet   = nmea_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = NULL,		/* iTrax doesn't support DGPS/WAAS/EGNOS */
-#ifdef ALLOW_CONTROLSEND
-    .control_send   = garmin_control_send,	/* send raw bytes */
-#endif /* ALLOW_CONTROLSEND */
+    .event_hook     = event_hook,	/* lietime event handler */
 #ifdef ALLOW_RECONFIGURE
-    .configurator   = itrax_configurator,/* set synchronous mode */
     .speed_switcher = itrax_speed,	/* how to change speeds */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = itrax_rate,	/* there's a sample-rate switcher */
     .min_cycle      = 0,		/* no hard limit */
-    .revert         = NULL,		/* no setting-reversion method */
 #endif /* ALLOW_RECONFIGURE */
-    .wrapup         = itrax_wrap,	/* sleep the receiver */
+#ifdef ALLOW_CONTROLSEND
+    .control_send   = garmin_control_send,	/* send raw bytes */
+#endif /* ALLOW_CONTROLSEND */
 };
 /*@ -redef @*/
 #endif /* ITRAX_ENABLE */
