@@ -1,4 +1,4 @@
-/* $Id: driver_proto.c 5412 2009-03-07 16:11:17Z esr $
+/* $Id: driver_proto.c 6566 2009-11-20 03:51:06Z esr $
  *
  * A prototype driver.  Doesn't run, doesn't even compile.
  *
@@ -21,11 +21,18 @@
  * driver author to find-and-replace to create a unique namespace for
  * driver functions.
  *
- * If using vi, ":%s/_PROTO_/MYDRIVER/g" and ":%s/_proto_/mydriver/g" should
- * produce a source file that comes very close to being useful. Other places
- * for new driver hooks are gpsd.h-tail, packet_states.h, libgpsd_core.c and
+ * If using vi, ":%s/_PROTO_/MYDRIVER/g" and ":%s/_proto_/mydriver/g"
+ * should produce a source file that comes very close to being useful.
+ * You will also need to add hooks for your new driver to:
+ * Makefile.am
+ * drivers.c
+ * gpsd.h-tail
+ * libgpsd_core.c
  * packet.c
+ * packet_states.h
  *
+ * see http://svn.berlios.de/viewvc/gpsd/trunk/?sortby=date&pathrev=5078
+ * for an example of how a new driver arrived.
  */
 
 #include <sys/types.h>
@@ -34,11 +41,12 @@
 #include <string.h>
 #include <math.h>
 #include <ctype.h>
+#ifndef S_SPLINT_S
 #include <unistd.h>
+#endif /* S_SPLINT_S */
 #include <time.h>
 #include <stdio.h>
 
-#include "gpsd_config.h"
 #include "gpsd.h"
 #if defined(_PROTO__ENABLE) && defined(BINARY_ENABLE)
 
@@ -57,15 +65,11 @@ static	gps_mask_t _proto__msg_svinfo(struct gps_device_t *, unsigned char *, siz
 /*
  * These methods may be called elsewhere in gpsd
  */
-static	ssize_t _proto__control_send(struct gps_device_t *, char *, size_t );
+static	ssize_t _proto__control_send(struct gps_device_t *, char *, size_t);
 static	bool _proto__probe_detect(struct gps_device_t *);
-static	void _proto__probe_wakeup(struct gps_device_t *);
-static	void _proto__probe_subtype(struct gps_device_t *, unsigned int );
-static	void _proto__configurator(struct gps_device_t *, unsigned int );
-static	bool _proto__set_speed(struct gps_device_t *, speed_t );
-static	void _proto__set_mode(struct gps_device_t *, int );
-static	void _proto__revert(struct gps_device_t *);
-static	void _proto__wrapup(struct gps_device_t *);
+static	void _proto__event_hook(struct gps_device_t *, event_t);
+static	bool _proto__set_speed(struct gps_device_t *, speed_t, char, int);
+static	void _proto__set_mode(struct gps_device_t *, int);
 
 /*
  * Decode the navigation solution message
@@ -94,19 +98,39 @@ _proto__msg_navsol(struct gps_device_t *session, unsigned char *buf, size_t data
     ecef_to_wgs84fix(&session->gpsdata, Px, Py, Pz, Vx, Vy, Vz);
     mask |= LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET  ;
 
-    session->gpsdata.fix.eph = GET_POSITION_ERROR();
+    session->gpsdata.fix.epx = GET_LONGITUDE_ERROR();
+    session->gpsdata.fix.epy = GET_LATITUDE_ERROR();
     session->gpsdata.fix.eps = GET_SPEED_ERROR();
     session->gpsdata.satellites_used = GET_SATELLITES_USED();
-    session->gpsdata.hdop = GET_HDOP();
-    session->gpsdata.vdop = GET_VDOP();
+    dop_clear(&session->gpsdata.dop);
+    session->gpsdata.dop.hdop = GET_HDOP();
+    session->gpsdata.dop.vdop = GET_VDOP();
     /* other DOP if available */
-    mask |= HDOP_SET | VDOP_SET | USED_SET;
+    mask |= DOP_SET;
 
     session->gpsdata.fix.mode = GET_FIX_MODE();
     session->gpsdata.status = GET_FIX_STATUS();
 
-    /* CYCLE_START_SET if this message starts a reporting period */
-    mask |= MODE_SET | STATUS_SET | CYCLE_START_SET ;
+    /*
+     * Mix in CLEAE_SET to clue the daemon in about when to clear fix
+     * information.  Mix in REPORT_SET when the sentence is reliably
+     * the last in a reporting cycle.
+     */
+    mask |= MODE_SET | STATUS_SET | REPORT_SET;
+
+    /* 
+     * At the end of each packet-cracking function, report at LOG_DATA level
+     * the fields it potentially set and the transfer mask. Doing this
+     * makes it relatively easy to track down data-management problems.
+     */
+    gpsd_report(LOG_DATA, "NAVSOL: time=%.2f, lat=%.2f lon=%.2f alt=%.2f mode=%d status=%d mask=%s\n",
+		session->gpsdata.fix.time,
+		session->gpsdata.fix.latitude,
+		session->gpsdata.fix.longitude,
+		session->gpsdata.fix.altitude,
+		session->gpsdata.fix.mode,
+		session->gpsdata.status,
+		gpsd_maskdump(mask));
 
     return mask;
 }
@@ -133,7 +157,7 @@ _proto__msg_utctime(struct gps_device_t *session, unsigned char *buf, size_t dat
     session->context->leap_seconds = GET_GPS_LEAPSECONDS();
 
     t = gpstime_to_unix(gps_week, tow/1000.0) - session->context->leap_seconds;
-    session->gpsdata.sentence_time = session->gpsdata.fix.time = t;
+    session->gpsdata.fix.time = t;
 
     return TIME_SET | ONLINE_SET;
 }
@@ -142,7 +166,7 @@ _proto__msg_utctime(struct gps_device_t *session, unsigned char *buf, size_t dat
  * GPS Satellite Info
  */
 static gps_mask_t
-_proto__msg_nav_svinfo(struct gps_device_t *session, unsigned char *buf, size_t data_len)
+_proto__msg_svinfo(struct gps_device_t *session, unsigned char *buf, size_t data_len)
 {
     unsigned char i, st, nchan, nsv;
     unsigned int tow;
@@ -171,7 +195,7 @@ _proto__msg_nav_svinfo(struct gps_device_t *session, unsigned char *buf, size_t
 	int off = GET_CHANNEL_STATUS(i);
 
 	session->gpsdata.PRN[i]		= PRN_THIS_CHANNEL_IS_TRACKING(i);
-	session->gpsdata.ss[i]		= SIGNAL_STRENGTH_FOR_CHANNEL(i);
+	session->gpsdata.ss[i]		= (float)SIGNAL_STRENGTH_FOR_CHANNEL(i);
 	session->gpsdata.elevation[i]	= SV_ELEVATION_FOR_CHANNEL(i);
 	session->gpsdata.azimuth[i]	= SV_AZIMUTH_FOR_CHANNEL(i);
 
@@ -181,8 +205,14 @@ _proto__msg_nav_svinfo(struct gps_device_t *session, unsigned char *buf, size_t
 	if(session->gpsdata.PRN[i])
 		st++;
     }
+    /* if the satellite-info setence gives you UTC time, use it */
+    session->gpsdata.skyview_time = NaN;
     session->gpsdata.satellites_used = nsv;
-    session->gpsdata.satellites = st;
+    session->gpsdata.satellites_visible = st;
+    gpsd_report(LOG_DATA, 
+		"SVINFO: visible=%d used=%d mask={SATELLITE|USED}\n",
+		session->gpsdata.satellites_visible, 
+		session->gpsdata.satellites_used);
     return SATELLITE_SET | USED_SET;
 }
 
@@ -193,11 +223,22 @@ _proto__msg_nav_svinfo(struct gps_device_t *session, unsigned char *buf, size_t
 gps_mask_t _proto__dispatch(struct gps_device_t *session, unsigned char *buf, size_t len)
 {
     size_t i;
-    int type, used, visible;
+    int type, used, visible, retmask = 0;
 
     if (len == 0)
 	return 0;
 
+    /*
+     * Set this if the driver reliably signals end of cycle.
+     * The core library zeroes it just before it calls each driver's
+     * packet analyzer.
+     */
+    session->cycle_end_reliable = true;
+    if (msgid == MY_START_OF_CYCLE)
+	retmask |= CLEAR_SET;
+    else if (msgid == MY_END_OF_CYCLE)
+	retmask |= REPORT_SET;
+
     type = GET_MESSAGE_TYPE();
 
     /* we may need to dump the raw packet */
@@ -246,29 +287,7 @@ static bool _proto__probe_detect(struct gps_device_t *session)
    return false;
 }
 
-static void _proto__probe_wakeup(struct gps_device_t *session)
-{
-   /*
-    * Code to make the device ready to communicate. This is
-    * run every time we are about to try a different baud
-    * rate in the autobaud sequence. Only needed if the
-    * device is in some kind of sleeping state. If a wakeup
-    * is not needed this method can be elided and the probe_wakeup
-    * member of the gps_type_t structure can be set to NULL.
-    */
-}
-
-static void _proto__probe_subtype(struct gps_device_t *session, unsigned int seq)
-{
-    /*
-     * Probe for subtypes here. If possible, get the software version and
-     * store it in session->subtype.  The seq values don't actually mean 
-     * anything, but conditionalizing probes on them gives the device 
-     * time to respond to each one.
-     */
-}
-
-#ifdef ALLOW_CONTROLSEND */
+#ifdef ALLOW_CONTROLSEND
 /**
  * Write data to the device, doing any required padding or checksumming
  */
@@ -283,7 +302,7 @@ static ssize_t _proto__control_send(struct gps_device_t *session,
    /* 
     * This copy to a public assembly buffer 
     * enables gpsmon to snoop the control message
-    * acter it has been sent.
+    * after it has been sent.
     */
    session->msgbuflen = msglen;
    (void)memcpy(session->msgbuf, msg, msglen);
@@ -297,14 +316,58 @@ static ssize_t _proto__control_send(struct gps_device_t *session,
 /*@ -charint +usedef +compdef @*/
 #endif /* ALLOW_CONTROLSEND */
 
-#ifdef ALLOW_CONFIGURE
-static void _proto__configurator(struct gps_device_t *session, unsigned int seq)
+#ifdef ALLOW_RECONFIGURE
+static void _proto__event_hook(struct gps_device_t *session, event_t event)
 {
-    /* 
-     * Change sentence mix and set reporting modes as needed.
-     * If your device has a default cycle time other than 1 second,
-     * set session->device->gpsdata.cycle here.
-     */
+    if (event == event_wakeup) {
+       /*
+	* Code to make the device ready to communicate. This is
+	* run every time we are about to try a different baud
+	* rate in the autobaud sequence. Only needed if the
+	* device is in some kind of sleeping state.
+	*/
+    }
+    if (event == event_identified) {
+	/*
+	 * Fires when the first full packet is recognized from a
+	 * previously unidentified device.  The session packet counter
+	 * is zeroed.  If your device has a default cycle time other
+	 * than 1 second, set session->device->gpsdata.cycle here. If
+	 * possible, get the software version and store it in
+	 * session->subtype.
+	 */
+    }
+    if (event == event_configure) {
+	/* 
+	 * Change sentence mix and set reporting modes as needed.
+	 * Called immediately after event_identified fires, then just
+	 * after every packet received thereafter, but you probably
+	 * only want to take actions on the first few packets after
+	 * the session packet counter has been zeroed,
+	 *
+	 * Remember that session->packet.counter is available when you
+	 * write this hook; you can use this fact to interleave configuration
+	 * sends with the first few packet reads, which is useful for
+	 * devices with small receive buffers.
+	 */
+    } else if (event == event_driver_switch) {
+	/*
+	 * Fires when the driver on a device is changed *after* it
+	 * has been identified.
+	 */
+    } else if (event == event_deactivate) {
+	/*
+	 * Fires when the device is deactivated.  Usr this to revert
+	 * whatever was done at event_identify and event_configure
+	 * time.
+	 */
+    } else if (event == event_reactivate) {
+       /*
+	* Fires when a device is reactivated after having been closed.  
+	* Use this hook for re-establishing device settings that
+	* it doesn't hold through closes.
+	*/
+    }
 }
 
 /*
@@ -316,7 +379,7 @@ static gps_mask_t _proto__parse_input(struct gps_device_t *session)
 {
     gps_mask_t st;
 
-    if (session->packet.type == _PROTO__PACKET){
+    if (session->packet.type == _PROTO__PACKET) {
 	st = _proto__dispatch(session, session->packet.outbuffer, session->packet.outbuflen);
 	session->gpsdata.driver_mode = MODE_BINARY;
 	return st;
@@ -359,21 +422,10 @@ static void _proto__set_mode(struct gps_device_t *session, int mode)
 	session->gpsdata.driver_mode = MODE_BINARY;
     }
 }
-
-static void _proto__revert(struct gps_device_t *session)
-{
-   /*
-    * Reverse what the .configurator method changed.
-    */
-}
-#endif /* ALLOW_CONFIGURE */
+#endif /* ALLOW_RECONFIGURE */
 
 static void _proto__wrapup(struct gps_device_t *session)
 {
-   /*
-    * Do release actions that are independent of whether the .configurator 
-    * method ran or not.
-    */
 }
 
 /* The methods in this code take parameters and have */
@@ -394,31 +446,23 @@ static void _proto__wrapup(struct gps_device_t *session)
 const struct gps_type_t _proto__binary = {
     /* Full name of type */
     .type_name        = "_proto_ binary",
-    /* associated lexer packet type */
-    .packet_type    = NMEA_PACKET;
+    /* Associated lexer packet type */
+    .packet_type      = _PROTO__PACKET,
     /* Response string that identifies device (not active) */
     .trigger          = NULL,
     /* Number of satellite channels supported by the device */
     .channels         = 12,
     /* Startup-time device detector */
     .probe_detect     = _proto__probe_detect,
-    /* Wakeup to be done before each baud hunt */
-    .probe_wakeup     = _proto__probe_wakeup,
-    /* Initialize the device and get subtype */
-    .probe_subtype    = _proto__probe_subtype,
     /* Packet getter (using default routine) */
     .get_packet       = generic_get,
     /* Parse message packets */
     .parse_packet     = _proto__parse_input,
     /* RTCM handler (using default routine) */
     .rtcm_writer      = pass_rtcm,
-#ifdef ALLOW_CONTROLSEND
-    /* Control string sender - should provide checksum and headers/trailer */
-    .control_send   = __proto__control_send,
-#endif /* ALLOW_CONTROLSEND */
+    /* fire on various lifetime events */
+    .event_hook       = _proto__event_hook,
 #ifdef ALLOW_RECONFIGURE
-    /* Enable what reports we need */
-    .configurator     = _proto__configurator,
     /* Speed (baudrate) switch */
     .speed_switcher   = _proto__set_speed,
     /* Switch to NMEA mode */
@@ -427,11 +471,11 @@ const struct gps_type_t _proto__binary = {
     .rate_switcher    = NULL,
     /* Minimum cycle time of the device */
     .min_cycle        = 1,
-    /* Undo the actions of .configurator */
-    .revert           = _proto__revert,
 #endif /* ALLOW_RECONFIGURE */
-    /* Puts device back to original settings */
-    .wrapup           = _proto__wrapup,
+#ifdef ALLOW_CONTROLSEND
+    /* Control string sender - should provide checksum and headers/trailer */
+    .control_send   = _proto__control_send,
+#endif /* ALLOW_CONTROLSEND */
 };
 #endif /* defined(_PROTO__ENABLE) && defined(BINARY_ENABLE) */
 
