@@ -1,4 +1,4 @@
-/* $Id: driver_zodiac.c 5410 2009-03-07 15:32:23Z esr $ */
+/* $Id: driver_zodiac.c 6908 2010-01-02 22:29:16Z esr $ */
 /*
  * Handle the Rockwell binary packet format supported by the old Zodiac chipset
  */
@@ -6,9 +6,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef S_SPLINT_S
 #include <unistd.h>
+#endif /* S_SPLINT_S */
 #include <math.h>
-#include "gpsd_config.h"
 #include "gpsd.h"
 
 #include "bits.h"
@@ -53,7 +54,7 @@ static int end_write(int fd, void *d, int len)
 }
 #else
 #define end_write write
-#endif
+#endif /* WORDS_BIGENDIAN */
 
 static ssize_t zodiac_spew(struct gps_device_t *session, unsigned short type, unsigned short *dat, int dlen)
 {
@@ -125,7 +126,9 @@ static ssize_t zodiac_send_rtcm(struct gps_device_t *session,
 #define getzlong(n)	getlongz(session->packet.outbuffer, n)
 
 static gps_mask_t handle1000(struct gps_device_t *session)
+/* time-position-velocity report */
 {
+    gps_mask_t mask;
     double subseconds;
     struct tm unpacked_date;
     /* ticks                      = getzlong(6); */
@@ -153,7 +156,7 @@ static gps_mask_t handle1000(struct gps_device_t *session)
     unpacked_date.tm_sec = (int)getzword(24);
     subseconds = (int)getzlong(25) / 1e9;
     /*@ -compdef */
-    session->gpsdata.fix.time = session->gpsdata.sentence_time =
+    session->gpsdata.fix.time =
 	(double)mkgmtime(&unpacked_date) + subseconds;
     /*@ +compdef */
 #ifdef NTPSHM_ENABLE
@@ -162,7 +165,7 @@ static gps_mask_t handle1000(struct gps_device_t *session)
      * tests and simple gps applications do not detect this. A live test
      * with the 1pps signal active is required. */
     if (session->context->enable_ntpshm && session->gpsdata.fix.mode > MODE_NO_FIX)
-	(void)ntpshm_put(session, session->gpsdata.fix.time + 1.1);
+	(void)ntpshm_put(session, session->gpsdata.fix.time, 1.1);
 #endif
     /*@ -type @*/
     session->gpsdata.fix.latitude  = ((long)getzlong(27)) * RAD_2_DEG * 1e-8;
@@ -181,8 +184,11 @@ static gps_mask_t handle1000(struct gps_device_t *session)
     session->mag_var		   = ((short)getzword(37)) * RAD_2_DEG * 1e-4;
     session->gpsdata.fix.climb     = ((short)getzword(38)) * 1e-2;
     /* map_datum                   = getzword(39); */
-    /* manual says these are 1-sigma */
-    session->gpsdata.fix.eph       = (int)getzlong(40) * 1e-2 * GPSD_CONFIDENCE;
+    /* 
+     * The manual says these are 1-sigma.  Device reports only eph, circular
+     * error; no harm in assigning it to both x and y components.
+     */
+    session->gpsdata.fix.epx = session->gpsdata.fix.epy = (int)getzlong(40) * 1e-2 * (1/sqrt(2)) * GPSD_CONFIDENCE;
     session->gpsdata.fix.epv       = (int)getzlong(42) * 1e-2 * GPSD_CONFIDENCE;
     session->gpsdata.fix.ept       = (int)getzlong(44) * 1e-2 * GPSD_CONFIDENCE;
     session->gpsdata.fix.eps       = (int)getzword(46) * 1e-2 * GPSD_CONFIDENCE;
@@ -214,22 +220,37 @@ static gps_mask_t handle1000(struct gps_device_t *session)
     gpsd_report(LOG_INF, "Separation: %f\n", getzword(33) * 1e-2);
 #endif
 
-    session->gpsdata.sentence_length = 55;
-    return TIME_SET|LATLON_SET|ALTITUDE_SET|CLIMB_SET|SPEED_SET|TRACK_SET|STATUS_SET|MODE_SET|CYCLE_START_SET; /* |HERR_SET|VERR_SET|SPEEDERR_SET */
+    mask = TIME_SET|LATLON_SET|ALTITUDE_SET|CLIMB_SET|SPEED_SET|TRACK_SET|STATUS_SET|MODE_SET;
+    gpsd_report(LOG_DATA, 
+		"1000: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f climb=%.2f mode=%d status=%d mask=%s\n",
+		session->gpsdata.fix.time,
+		session->gpsdata.fix.latitude,
+		session->gpsdata.fix.longitude,
+		session->gpsdata.fix.altitude,
+		session->gpsdata.fix.track,
+		session->gpsdata.fix.speed,
+		session->gpsdata.fix.climb,
+		session->gpsdata.fix.mode,
+		session->gpsdata.status,
+		gpsd_maskdump(mask));
+    return mask;
 }
 
 static gps_mask_t handle1002(struct gps_device_t *session)
+/* satellite signal quality report */
 {
     int i, j, status, prn;
 
-    session->gpsdata.satellites_used = 0;
-    memset(session->gpsdata.used,0,sizeof(session->gpsdata.used));
     /* ticks                      = getzlong(6); */
     /* sequence                   = getzword(8); */
     /* measurement_sequence       = getzword(9); */
-    /* gps_week                   = getzword(10); */
-    /* gps_seconds                = getzlong(11); */
+    /*@+charint@*/
+    int gps_week                   = getzword(10);
+    int gps_seconds                = getzlong(11);
     /* gps_nanoseconds            = getzlong(13); */
+    /*@-charint@*/
+    session->gpsdata.satellites_used = 0;
+    memset(session->gpsdata.used,0,sizeof(session->gpsdata.used));
     for (i = 0; i < ZODIAC_CHANNELS; i++) {
 	/*@ -type @*/
 	session->driver.zodiac.Zv[i] = status = (int)getzword(15 + (3 * i));
@@ -249,28 +270,42 @@ static gps_mask_t handle1002(struct gps_device_t *session)
 	for (j = 0; j < ZODIAC_CHANNELS; j++) {
 	    if (session->gpsdata.PRN[j] != prn)
 		continue;
-	    session->gpsdata.ss[j] = (int)getzword(17 + (3 * i));
+	    session->gpsdata.ss[j] = (float)getzword(17 + (3 * i));
 	    break;
 	}
     }
+    session->gpsdata.skyview_time = gpstime_to_unix(gps_week, (double)gps_seconds);
+    gpsd_report(LOG_DATA, 
+		"1002: visible=%d used=%d mask={SATELLITE|USED}\n",
+		session->gpsdata.satellites_visible, 
+		session->gpsdata.satellites_used);
     return SATELLITE_SET | USED_SET;
 }
 
 static gps_mask_t handle1003(struct gps_device_t *session)
+/* skyview report */
 {
-    int i;
+    int i, n;
+
+    /* The Polaris (and probably the DAGR) emit some strange variant of
+     * this message which causes gpsd to crash filtering on impossible
+     * number of satellites avoids this */
+    n = (int)getzword(14);
+    if ((n < 0) || (n >12))
+	    return 0;
 
     /* ticks              = getzlong(6); */
     /* sequence           = getzword(8); */
-    session->gpsdata.gdop = (unsigned int)getzword(9) * 1e-2;
-    session->gpsdata.pdop = (unsigned int)getzword(10) * 1e-2;
-    session->gpsdata.hdop = (unsigned int)getzword(11) * 1e-2;
-    session->gpsdata.vdop = (unsigned int)getzword(12) * 1e-2;
-    session->gpsdata.tdop = (unsigned int)getzword(13) * 1e-2;
-    session->gpsdata.satellites = (int)getzword(14);
+    clear_dop(&session->gpsdata.dop);
+    session->gpsdata.dop.gdop = (unsigned int)getzword(9) * 1e-2;
+    session->gpsdata.dop.pdop = (unsigned int)getzword(10) * 1e-2;
+    session->gpsdata.dop.hdop = (unsigned int)getzword(11) * 1e-2;
+    session->gpsdata.dop.vdop = (unsigned int)getzword(12) * 1e-2;
+    session->gpsdata.dop.tdop = (unsigned int)getzword(13) * 1e-2;
+    session->gpsdata.satellites_visible = n;
 
     for (i = 0; i < ZODIAC_CHANNELS; i++) {
-	if (i < session->gpsdata.satellites) {
+	if (i < session->gpsdata.satellites_visible) {
 	    session->gpsdata.PRN[i] = (int)getzword(15 + (3 * i));
 	    session->gpsdata.azimuth[i] = (int)(((short)getzword(16 + (3 * i))) * RAD_2_DEG * 1e-4);
 	    if (session->gpsdata.azimuth[i] < 0)
@@ -286,10 +321,20 @@ static gps_mask_t handle1003(struct gps_device_t *session)
 	    session->gpsdata.elevation[i] = 0;
 	}
     }
-    return SATELLITE_SET | HDOP_SET | VDOP_SET | PDOP_SET;
+    session->gpsdata.skyview_time = NAN;
+    gpsd_report(LOG_DATA, "NAVDOP: visible=%d gdop=%.2f pdop=%.2f "
+		"hdop=%.2f vdop=%.2f tdop=%.2f mask={SATELLITE|DOP}\n",
+		session->gpsdata.satellites_visible,
+		session->gpsdata.dop.gdop,
+		session->gpsdata.dop.hdop,
+		session->gpsdata.dop.vdop,
+		session->gpsdata.dop.pdop,
+		session->gpsdata.dop.tdop);
+    return SATELLITE_SET | DOP_SET;
 }
 
 static void handle1005(struct gps_device_t *session UNUSED)
+/* fix quality report */
 {
     /* ticks              = getzlong(6); */
     /* sequence           = getzword(8); */
@@ -323,6 +368,7 @@ static void handle1005(struct gps_device_t *session UNUSED)
 }
 
 static gps_mask_t handle1011(struct gps_device_t *session)
+/* version report */
 {
     /*
      * This is UNTESTED -- but harmless if buggy.  Added to support
@@ -332,12 +378,13 @@ static gps_mask_t handle1011(struct gps_device_t *session)
     getstringz(session->subtype,
 	      session->packet.outbuffer,
 	      19, 28);	/* software version field */
-    gpsd_report(LOG_INF, "Software version: %s\n", session->subtype);
+    gpsd_report(LOG_DATA, "1011: subtype=%s mask={DEVICEID}\n", session->subtype);
     return DEVICEID_SET;
 }
 
 
 static void handle1108(struct gps_device_t *session)
+/* leap-second correction report */
 {
     /* ticks              = getzlong(6); */
     /* sequence           = getzword(8); */
@@ -381,7 +428,6 @@ static gps_mask_t zodiac_analyze(struct gps_device_t *session)
     for (i = 0; i < (int)session->packet.outbuflen; i++)
 	(void)snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf),
 		       "%02x", (unsigned int)session->packet.outbuffer[i]);
-    (void)strlcat(buf, "\n", BUFSIZ);
     gpsd_report(LOG_RAW, "Raw Zodiac packet type %d length %zd: %s\n",
 		id, session->packet.outbuflen, buf);
 
@@ -390,9 +436,16 @@ static gps_mask_t zodiac_analyze(struct gps_device_t *session)
 
     (void)snprintf(session->gpsdata.tag,sizeof(session->gpsdata.tag),"%u",id);
 
+    /* 
+     * Normal cycle for these devices is 1001 1002.
+     * We count 1001 as end of cycle because 1002 doesn't
+     * carry fix information.
+     */
+    session->cycle_end_reliable = true;
+
     switch (id) {
     case 1000:
-	return handle1000(session);
+	return handle1000(session) | (CLEAR_SET | REPORT_SET);
     case 1002:
 	return handle1002(session);
     case 1003:
@@ -468,24 +521,20 @@ const struct gps_type_t zodiac_binary =
     .packet_type    = ZODIAC_PACKET,	/* associated lexer packet type */
     .trigger	    = NULL,		/* no trigger */
     .channels       = 12,		/* consumer-grade GPS */
-    .probe_wakeup   = NULL,		/* no probe on baud rate change */
     .probe_detect   = NULL,		/* no probe */
-    .probe_subtype  = NULL,		/* no initialization */
     .get_packet     = generic_get,	/* use the generic packet getter */
     .parse_packet   = zodiac_analyze,	/* parse message packets */
     .rtcm_writer    = zodiac_send_rtcm,	/* send DGPS correction */
-#ifdef ALLOW_CONTROLSEND
-    .control_send   = zodiac_control_send,	/* for gpsctl and friends */
-#endif /* ALLOW_CONTROLSEND */
+    .event_hook     = NULL,		/* no configuration */
 #ifdef ALLOW_RECONFIGURE
-    .configurator   = NULL,		/* no configuration */
     .speed_switcher = zodiac_speed_switch,/* we can change baud rate */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .min_cycle      = 1,		/* not relevant, no rate switch */
-    .revert	    = NULL,		/* no reversion hook */
 #endif /* ALLOW_RECONFIGURE */
-    .wrapup	    = NULL,		/* caller might supply a close hook */
+#ifdef ALLOW_CONTROLSEND
+    .control_send   = zodiac_control_send,	/* for gpsctl and friends */
+#endif /* ALLOW_CONTROLSEND */
 };
 
 #endif /* ZODIAC_ENABLE */
