@@ -1,15 +1,16 @@
-/* $Id: gpsutils.c 4900 2009-01-05 21:47:53Z esr $ */
+/* $Id: gpsutils.c 7007 2010-02-27 23:46:26Z ckuethe $ */
 /* gpsutils.c -- code shared between low-level and high-level interfaces */
 #include <sys/types.h>
 #include <stdio.h>
+#ifndef S_SPLINT_S
 #include <unistd.h>
+#endif /* S_SPLINT_S */
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
 #include <stdarg.h>
 #include <time.h>
 
-#include "gpsd_config.h"
 #include "gpsd.h"
 
 #define MONTHSPERYEAR	12		/* months per calendar year */
@@ -25,13 +26,15 @@ void gps_clear_fix(/*@out@*/struct gps_fix_t *fixp)
     fixp->climb = NAN;
     fixp->altitude = NAN;
     fixp->ept = NAN;
-    fixp->eph = NAN;
+    fixp->epx = NAN;
+    fixp->epy = NAN;
     fixp->epv = NAN;
     fixp->epd = NAN;
     fixp->eps = NAN;
     fixp->epc = NAN;
 }
 
+#ifdef __UNUSED__
 unsigned int gps_valid_fields(/*@in@*/struct gps_fix_t *fixp)
 {
     unsigned int valid = 0;
@@ -52,7 +55,7 @@ unsigned int gps_valid_fields(/*@in@*/struct gps_fix_t *fixp)
 	valid |= CLIMB_SET;
     if (isnan(fixp->ept) == 0)
 	valid |= TIMERR_SET;
-    if (isnan(fixp->eph) == 0)
+    if (isnan(fixp->epx) == 0 && isnan(fixp->epy) == 0)
 	valid |= HERR_SET;
     if (isnan(fixp->epv) == 0)
 	valid |= VERR_SET;
@@ -99,6 +102,7 @@ char *gps_show_transfer(int transfer)
     return showbuf;
 /*@ +statictrans @*/
 }
+#endif /* __UNUSED__ */
 
 void gps_merge_fix(/*@ out @*/struct gps_fix_t *to,
 		   gps_mask_t transfer,
@@ -125,8 +129,10 @@ void gps_merge_fix(/*@ out @*/struct gps_fix_t *to,
 	to->climb = from->climb;
     if ((transfer & TIMERR_SET)!=0)
 	to->ept = from->ept;
-    if ((transfer & HERR_SET)!=0)
-	to->eph = from->eph;
+    if ((transfer & HERR_SET)!=0) {
+	to->epx = from->epx;
+	to->epy = from->epy;
+    }
     if ((transfer & VERR_SET)!=0)
 	to->epv = from->epv;
     if ((transfer & SPEEDERR_SET)!=0)
@@ -135,10 +141,10 @@ void gps_merge_fix(/*@ out @*/struct gps_fix_t *to,
 	to->epc = from->epc;
 }
 
-double timestamp(void) 
+double timestamp(void)
 {
-    struct timeval tv; 
-    (void)gettimeofday(&tv, NULL); 
+    struct timeval tv;
+    (void)gettimeofday(&tv, NULL);
     /*@i1@*/return(tv.tv_sec + tv.tv_usec*1e-6);
 }
 
@@ -201,14 +207,14 @@ double iso8601_to_unix(/*@in@*/char *isotime)
 
     (void)strftime(timestr, sizeof(timestr), "%Y-%m-%dT%H:%M:%S", &when);
     (void)snprintf(fractstr, sizeof(fractstr), "%.1f", fractional);
-    /* add fractional part, ignore leading 0; "0.2" -> ".2" */ 
+    /* add fractional part, ignore leading 0; "0.2" -> ".2" */
     (void)snprintf(isotime, len, "%s%sZ", timestr, fractstr+1);
     return isotime;
 }
 
 /*
- * The 'week' part of GPS dates are specified in weeks since 0000 on 06 
- * January 1980, with a rollover at 1024.  At time of writing the last 
+ * The 'week' part of GPS dates are specified in weeks since 0000 on 06
+ * January 1980, with a rollover at 1024.  At time of writing the last
  * rollover happened at 0000 22 August 1999.  Time-of-week is in seconds.
  *
  * This code copes with both conventional GPS weeks and the "extended"
@@ -248,56 +254,74 @@ void unix_to_gpstime(double unixtime, /*@out@*/int *week, /*@out@*/double *tow)
 
 #define Deg2Rad(n)	((n) * DEG_2_RAD)
 
-static double CalcRad(double lat)
-/* earth's radius of curvature in meters at specified latitude.*/
-{
-    const double a = 6378.137;
-    const double e2 = 0.081082 * 0.081082;
-    // the radius of curvature of an ellipsoidal Earth in the plane of a
-    // meridian of latitude is given by
-    //
-    // R' = a * (1 - e^2) / (1 - e^2 * (sin(lat))^2)^(3/2)
-    //
-    // where a is the equatorial radius,
-    // b is the polar radius, and
-    // e is the eccentricity of the ellipsoid = sqrt(1 - b^2/a^2)
-    //
-    // a = 6378 km (3963 mi) Equatorial radius (surface to center distance)
-    // b = 6356.752 km (3950 mi) Polar radius (surface to center distance)
-    // e = 0.081082 Eccentricity
-    double sc = sin(Deg2Rad(lat));
-    double x = a * (1.0 - e2);
-    double z = 1.0 - e2 * sc * sc;
-    double y = pow(z, 1.5);
-    double r = x / y;
-
-    return r * 1000.0;	// Convert to meters
-}
-
 double earth_distance(double lat1, double lon1, double lat2, double lon2)
 /* distance in meters between two points specified in degrees. */
 {
-    double x1 = CalcRad(lat1) * cos(Deg2Rad(lon1)) * sin(Deg2Rad(90-lat1));
-    double x2 = CalcRad(lat2) * cos(Deg2Rad(lon2)) * sin(Deg2Rad(90-lat2));
-    double y1 = CalcRad(lat1) * sin(Deg2Rad(lon1)) * sin(Deg2Rad(90-lat1));
-    double y2 = CalcRad(lat2) * sin(Deg2Rad(lon2)) * sin(Deg2Rad(90-lat2));
-    double z1 = CalcRad(lat1) * cos(Deg2Rad(90-lat1));
-    double z2 = CalcRad(lat2) * cos(Deg2Rad(90-lat2));
-    double a = (x1*x2 + y1*y2 + z1*z2)/pow(CalcRad((lat1+lat2)/2),2);
-    // a should be in [1, -1] but can sometimes fall outside it by
-    // a very small amount due to rounding errors in the preceding
-    // calculations.  This is prone to happen when the argument points
-    // are very close together.  Return NaN so calculations trying
-    // to use this will also blow up.
-    if (fabs(a) > 1) 
-	return NAN;
-    else
-	return CalcRad((lat1+lat2) / 2) * acos(a);
+	/*
+	 * this is a translation of the javascript implementation of the
+	 * Vincenty distance formula by Chris Veness. See
+	 * http://www.movable-type.co.uk/scripts/latlong-vincenty.html
+	 */
+	double a, b, f; // WGS-84 ellipsoid params
+	double L, L_P, U1, U2, s_U1, c_U1, s_U2, c_U2;
+	double uSq, A, B, d_S, lambda;
+	double s_L, c_L, s_S, C;
+	double c_S, S, s_A, c_SqA, c_2SM;
+	int i = 100;
+
+	a = WGS84A;
+	b = WGS84B;
+	f = 1/WGS84F;
+	L = Deg2Rad(lon2 - lon1);
+	U1 = atan((1 - f) * tan(Deg2Rad(lat1)));
+	U2 = atan((1 - f) * tan(Deg2Rad(lat2)));
+	s_U1 = sin(U1);
+	c_U1 = cos(U1);
+	s_U2 = sin(U2);
+	c_U2 = cos(U2);
+	lambda = L;
+
+	do {
+		s_L = sin(lambda);
+		c_L = cos(lambda);
+		s_S = sqrt((c_U2 * s_L) * (c_U2 * s_L) +
+			   (c_U1 * s_U2 - s_U1 * c_U2 * c_L) *
+			   (c_U1 * s_U2 - s_U1 * c_U2 * c_L));
+
+		if (s_S==0)
+			return 0;
+
+		c_S = s_U1 * s_U2 + c_U1 * c_U2 * c_L;
+		S = atan2(s_S, c_S);
+		s_A = c_U1 * c_U2 * s_L / s_S;
+		c_SqA = 1 - s_A * s_A;
+		c_2SM = c_S - 2 * s_U1 * s_U2 / c_SqA;
+
+		if (isnan(c_2SM))
+			c_2SM = 0;
+
+		C = f / 16 * c_SqA * (4 + f * (4 - 3 * c_SqA));
+		L_P = lambda;
+		lambda = L + (1 - C) * f * s_A *
+		    (S + C * s_S * (c_2SM + C * c_S * (2 * c_2SM * c_2SM - 1)));
+	} while ((fabs(lambda - L_P) > 1.0e-12) && (--i > 0));
+
+	if (i == 0)
+		return NAN; // formula failed to converge
+
+	uSq = c_SqA * ((a*a) - (b*b)) / (b*b);
+	A = 1 + uSq / 16384 * (4096 + uSq * (-768 + uSq * (320 - 175 * uSq)));
+	B = uSq / 1024 * (256 + uSq * (-128 + uSq * (74 - 47 * uSq)));
+	d_S = B * s_S * (c_2SM + B/4 *
+			(c_S * (-1 + 2 * c_2SM * c_2SM) - B/6 * c_2SM *
+			(-3 + 4 * s_S * s_S) * (-3 + 4 * c_2SM * c_2SM)));
+
+	return(WGS84B * A * (S - d_S));
 }
 
 /*****************************************************************************
 
-Carl Carter of SiRF supplied this algorithm for computing DOPs from 
+Carl Carter of SiRF supplied this algorithm for computing DOPs from
 a list of visible satellites (some typos corrected)...
 
 For satellite n, let az(n) = azimuth angle from North and el(n) be elevation.
@@ -340,11 +364,11 @@ Here's how we implement it...
 
 First, each compute element P(i,j) of the 4x4 product A~*A.
 If S(k=1,k=n): f(...) is the sum of f(...) as k varies from 1 to n, then
-applying the definition of matrix product tells us: 
+applying the definition of matrix product tells us:
 
 P(i,j) = S(k=1,k=n): B(i, k) * A(k, j)
 
-But because B is the transpose of A, this reduces to 
+But because B is the transpose of A, this reduces to
 
 P(i,j) = S(k=1,k=n): A(k, i) * A(k, j)
 
@@ -383,6 +407,11 @@ driver.
 
 ******************************************************************************/
 
+void clear_dop(/*@out@*/struct dop_t *dop)
+{
+    dop->xdop = dop->ydop = dop->vdop = dop->tdop = dop->hdop = dop->pdop = dop->gdop = NAN;
+}
+
 /*@ -fixedformalarray -mustdefine @*/
 static bool invert(double mat[4][4], /*@out@*/double inverse[4][4])
 {
@@ -396,9 +425,9 @@ static bool invert(double mat[4][4], /*@out@*/double inverse[4][4])
   double Det2_13_01 = mat[1][0]*mat[3][1] - mat[1][1]*mat[3][0];
   //double Det2_13_02 = mat[1][0]*mat[3][2] - mat[1][2]*mat[3][0];
   double Det2_13_03 = mat[1][0]*mat[3][3] - mat[1][3]*mat[3][0];
-  //double Det2_13_12 = mat[1][1]*mat[3][2] - mat[1][2]*mat[3][1];  
+  //double Det2_13_12 = mat[1][1]*mat[3][2] - mat[1][2]*mat[3][1]; 
   double Det2_13_13 = mat[1][1]*mat[3][3] - mat[1][3]*mat[3][1];
-  //double Det2_13_23 = mat[1][2]*mat[3][3] - mat[1][3]*mat[3][2];  
+  //double Det2_13_23 = mat[1][2]*mat[3][3] - mat[1][3]*mat[3][2]; 
   double Det2_23_01 = mat[2][0]*mat[3][1] - mat[2][1]*mat[3][0];
   double Det2_23_02 = mat[2][0]*mat[3][2] - mat[2][2]*mat[3][0];
   double Det2_23_03 = mat[2][0]*mat[3][3] - mat[2][3]*mat[3][0];
@@ -407,15 +436,15 @@ static bool invert(double mat[4][4], /*@out@*/double inverse[4][4])
   double Det2_23_23 = mat[2][2]*mat[3][3] - mat[2][3]*mat[3][2];
 
   // Find all NECESSARY 3x3 subdeterminants
-  double Det3_012_012 = mat[0][0]*Det2_12_12 - mat[0][1]*Det2_12_02 
-  				+ mat[0][2]*Det2_12_01;
-  //double Det3_012_013 = mat[0][0]*Det2_12_13 - mat[0][1]*Det2_12_03 
+  double Det3_012_012 = mat[0][0]*Det2_12_12 - mat[0][1]*Det2_12_02
+				+ mat[0][2]*Det2_12_01;
+  //double Det3_012_013 = mat[0][0]*Det2_12_13 - mat[0][1]*Det2_12_03
   //				+ mat[0][3]*Det2_12_01;
   //double Det3_012_023 = mat[0][0]*Det2_12_23 - mat[0][2]*Det2_12_03
   //				+ mat[0][3]*Det2_12_02;
-  //double Det3_012_123 = mat[0][1]*Det2_12_23 - mat[0][2]*Det2_12_13 
+  //double Det3_012_123 = mat[0][1]*Det2_12_23 - mat[0][2]*Det2_12_13
   //				+ mat[0][3]*Det2_12_12;
-  //double Det3_013_012 = mat[0][0]*Det2_13_12 - mat[0][1]*Det2_13_02 
+  //double Det3_013_012 = mat[0][0]*Det2_13_12 - mat[0][1]*Det2_13_02
   //				+ mat[0][2]*Det2_13_01;
   double Det3_013_013 = mat[0][0]*Det2_13_13 - mat[0][1]*Det2_13_03
 				+ mat[0][3]*Det2_13_01;
@@ -423,7 +452,7 @@ static bool invert(double mat[4][4], /*@out@*/double inverse[4][4])
   //				+ mat[0][3]*Det2_13_02;
   //double Det3_013_123 = mat[0][1]*Det2_13_23 - mat[0][2]*Det2_13_13
   //				+ mat[0][3]*Det2_13_12;
-  //double Det3_023_012 = mat[0][0]*Det2_23_12 - mat[0][1]*Det2_23_02 
+  //double Det3_023_012 = mat[0][0]*Det2_23_12 - mat[0][1]*Det2_23_02
   //				+ mat[0][2]*Det2_23_01;
   //double Det3_023_013 = mat[0][0]*Det2_23_13 - mat[0][1]*Det2_23_03
   //				+ mat[0][3]*Det2_23_01;
@@ -431,20 +460,20 @@ static bool invert(double mat[4][4], /*@out@*/double inverse[4][4])
 				+ mat[0][3]*Det2_23_02;
   //double Det3_023_123 = mat[0][1]*Det2_23_23 - mat[0][2]*Det2_23_13
   //				+ mat[0][3]*Det2_23_12;
-  double Det3_123_012 = mat[1][0]*Det2_23_12 - mat[1][1]*Det2_23_02 
+  double Det3_123_012 = mat[1][0]*Det2_23_12 - mat[1][1]*Det2_23_02
 				+ mat[1][2]*Det2_23_01;
-  double Det3_123_013 = mat[1][0]*Det2_23_13 - mat[1][1]*Det2_23_03 
+  double Det3_123_013 = mat[1][0]*Det2_23_13 - mat[1][1]*Det2_23_03
 				+ mat[1][3]*Det2_23_01;
-  double Det3_123_023 = mat[1][0]*Det2_23_23 - mat[1][2]*Det2_23_03 
+  double Det3_123_023 = mat[1][0]*Det2_23_23 - mat[1][2]*Det2_23_03
 				+ mat[1][3]*Det2_23_02;
-  double Det3_123_123 = mat[1][1]*Det2_23_23 - mat[1][2]*Det2_23_13 
+  double Det3_123_123 = mat[1][1]*Det2_23_23 - mat[1][2]*Det2_23_13
 				+ mat[1][3]*Det2_23_12;
 
   // Find the 4x4 determinant
   static double det;
-          det =   mat[0][0]*Det3_123_123 
-		- mat[0][1]*Det3_123_023 
-		+ mat[0][2]*Det3_123_013 
+	  det =   mat[0][0]*Det3_123_123
+		- mat[0][1]*Det3_123_023
+		+ mat[0][2]*Det3_123_013
 		- mat[0][3]*Det3_123_012;
 
   // Very small determinants probably reflect floating-point fuzz near zero
@@ -472,16 +501,15 @@ static bool invert(double mat[4][4], /*@out@*/double inverse[4][4])
   inverse[3][3] =  Det3_012_012 / det;
 
   return true;
-}  
+}
 /*@ +fixedformalarray +mustdefine @*/
 
-gps_mask_t dop(struct gps_data_t *gpsdata)
+gps_mask_t fill_dop(const struct gps_data_t *gpsdata, struct dop_t *dop)
 {
     double prod[4][4];
     double inv[4][4];
     double satpos[MAXCHANNELS][4];
-    double hdop, vdop, pdop, tdop, gdop;
-    gps_mask_t mask;
+    double xdop, ydop, hdop, vdop, pdop, tdop, gdop;
     int i, j, k, n;
 
 #ifdef __UNUSED__
@@ -514,12 +542,12 @@ gps_mask_t dop(struct gps_data_t *gpsdata)
 #endif /* __UNUSED__ */
 
     for (i = 0; i < 4; ++i) { //< rows
-        for (j = 0; j < 4; ++j) { //< cols
-            prod[i][j] = 0.0;
-            for (k = 0; k < n; ++k) {
-                prod[i][j] += satpos[k][i] * satpos[k][j];
-            }
-        }
+	for (j = 0; j < 4; ++j) { //< cols
+	    prod[i][j] = 0.0;
+	    for (k = 0; k < n; ++k) {
+		prod[i][j] += satpos[k][i] * satpos[k][j];
+	    }
+	}
     }
 
 #ifdef __UNUSED__
@@ -541,50 +569,54 @@ gps_mask_t dop(struct gps_data_t *gpsdata)
 	    gpsd_report(LOG_RAW, "%f %f %f %f\n",
 			inv[k][0], inv[k][1], inv[k][2], inv[k][3]);
 	}
-	gpsd_report(LOG_INF, "HDOP: reported = %f, computed = %f\n",
-		    gpsdata->hdop, sqrt(inv[0][0] + inv[1][1]));
 #endif /* __UNUSED__ */
     } else {
-	gpsd_report(LOG_WARN, "LOS matrix is singular, can't calculate DOPs.\n");
+	gpsd_report(LOG_DATA,
+	    "LOS matrix is singular, can't calculate DOPs - source '%s'\n",
+	    gpsdata->dev.path);
 	return 0;
     }
 
+    xdop = sqrt(inv[0][0]);
+    ydop = sqrt(inv[1][1]);
     hdop = sqrt(inv[0][0] + inv[1][1]);
     vdop = sqrt(inv[2][2]);
     pdop = sqrt(inv[0][0] + inv[1][1] + inv[2][2]);
     tdop = sqrt(inv[3][3]);
     gdop = sqrt(inv[0][0] + inv[1][1] + inv[2][2] + inv[3][3]);
-    mask = 0;
 
-    gpsd_report(LOG_PROG, "DOPS computed/reported: H=%f/%f, V=%f/%f, P=%f/%f, T=%f/%f, G=%f/%f\n",
-		hdop, gpsdata->hdop,
-		vdop, gpsdata->vdop,
-		pdop, gpsdata->pdop,
-		tdop, gpsdata->tdop,
-		gdop, gpsdata->gdop);
+    gpsd_report(LOG_DATA, "DOPS computed/reported: X=%f/%f, Y=%f/%f, H=%f/%f, V=%f/%f, P=%f/%f, T=%f/%f, G=%f/%f\n",
+		xdop, dop->xdop,
+		ydop, dop->ydop,
+		hdop, dop->hdop,
+		vdop, dop->vdop,
+		pdop, dop->pdop,
+		tdop, dop->tdop,
+		gdop, dop->gdop);
 
     /*@ -usedef @*/
-    if (isnan(gpsdata->hdop)!=0) {
-	gpsdata->hdop = hdop;
-	mask |= HDOP_SET;
+    if (isnan(dop->xdop)!=0) {
+	dop->xdop = xdop;
     }
-    if (isnan(gpsdata->vdop)!=0) {
-	gpsdata->vdop = vdop;
-	mask |= VDOP_SET;
+    if (isnan(dop->ydop)!=0) {
+	dop->ydop = ydop;
     }
-    if (isnan(gpsdata->pdop)!=0) {
-	gpsdata->pdop = pdop;
-	mask |= PDOP_SET;
+    if (isnan(dop->hdop)!=0) {
+	dop->hdop = hdop;
     }
-    if (isnan(gpsdata->tdop)!=0) {
-	gpsdata->tdop = tdop;
-	mask |= TDOP_SET;
+    if (isnan(dop->vdop)!=0) {
+	dop->vdop = vdop;
     }
-    if (isnan(gpsdata->gdop)!=0) {
-	gpsdata->gdop = gdop;
-	mask |= GDOP_SET;
+    if (isnan(dop->pdop)!=0) {
+	dop->pdop = pdop;
+    }
+    if (isnan(dop->tdop)!=0) {
+	dop->tdop = tdop;
+    }
+    if (isnan(dop->gdop)!=0) {
+	dop->gdop = gdop;
     }
     /*@ +usedef @*/
 
-    return mask;
+    return DOP_SET;
 }
