@@ -1,9 +1,12 @@
-/* $Id: gpxlogger.c 5326 2009-03-02 23:24:03Z esr $ */
+/* $Id: gpxlogger.c 6920 2010-01-12 19:22:47Z esr $ */
+#include <stdlib.h>
+#include "gpsd_config.h"
 #include <sys/types.h>
 #include <string.h>
-#include <stdlib.h>
 #include <stdio.h>
+#ifdef HAVE_SYSLOG_H
 #include <syslog.h>
+#endif /* HAVE_SYSLOG_H */
 #include <math.h>
 #include <time.h>
 #include <signal.h>
@@ -12,8 +15,6 @@
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #endif /* S_SPLINT_S */
-
-#include "gpsd_config.h"
 #include "gps.h"
 #include "gpsdclient.h"
 
@@ -31,6 +32,9 @@ static time_t int_time, old_int_time;
 static bool intrack = false;
 static bool first = true;
 static time_t timeout = 5; /* seconds */
+#ifdef CLIENTDEBUG_ENABLE
+static int debug;
+#endif /* CLIENTDEBUG_ENABLE */
 
 static void print_gpx_header(void) 
 {
@@ -89,7 +93,7 @@ static void print_fix(struct gps_fix_t *fix, struct tm *time)
      * we don't necessarily have access to some of the stuff in gsdata.
      * Might mean some of this stuff should be promoted.
      */
-    if ((gpsdata->status >= 2) && (gpsdata->fix.mode >= MODE_3D)){
+    if ((gpsdata->status >= 2) && (gpsdata->fix.mode >= MODE_3D)) {
 	/* dgps or pps */
 	if (gpsdata->fix.mode == 4) { /* military pps */
 	    (void)printf("        <fix>pps</fix>\n");
@@ -172,13 +176,21 @@ static struct gps_fix_t gpsfix;
 
 #include <glib/gprintf.h>
 
+/*
+ * FIXME: use here is a minor bug, should report epx and epy separately.
+ * How to mix together epx and epy to get a horizontal circular error.
+ */
+#define EMIX(x, y)	(((x) > (y)) ? (x) : (y))
+
 DBusConnection* connection;
 
-static char devname[BUFSIZ];
+static char gpsd_devname[BUFSIZ];
 
-static DBusHandlerResult handle_gps_fix (DBusMessage* message) 
+static DBusHandlerResult handle_gps_fix (DBusMessage* message)
 {
     DBusError	error;
+    /* this packet format was designed before we split eph */
+    double eph = EMIX(gpsfix.epx, gpsfix.epy);
 
     dbus_error_init (&error);
 
@@ -189,7 +201,7 @@ static DBusHandlerResult handle_gps_fix (DBusMessage* message)
 			   DBUS_TYPE_DOUBLE, &gpsfix.ept,
 			   DBUS_TYPE_DOUBLE, &gpsfix.latitude,
 			   DBUS_TYPE_DOUBLE, &gpsfix.longitude,
-			   DBUS_TYPE_DOUBLE, &gpsfix.eph,
+			   DBUS_TYPE_DOUBLE, &eph,
 			   DBUS_TYPE_DOUBLE, &gpsfix.altitude,
 			   DBUS_TYPE_DOUBLE, &gpsfix.epv,
 			   DBUS_TYPE_DOUBLE, &gpsfix.track,
@@ -198,9 +210,9 @@ static DBusHandlerResult handle_gps_fix (DBusMessage* message)
 			   DBUS_TYPE_DOUBLE, &gpsfix.eps,
 			   DBUS_TYPE_DOUBLE, &gpsfix.climb,
 			   DBUS_TYPE_DOUBLE, &gpsfix.epc,
-			   DBUS_TYPE_STRING, &devname,
+			   DBUS_TYPE_STRING, &gpsd_devname,
 			   DBUS_TYPE_INVALID);
-	
+
     conditionally_log_fix(&gpsfix);
     return DBUS_HANDLER_RESULT_HANDLED;
 }
@@ -264,11 +276,14 @@ static int dbus_mainloop(void)
  **************************************************************************/
 
 struct fixsource_t source;
-static int casoc = 0;
 
 static void process(struct gps_data_t *gpsdata,
-	     char *buf UNUSED, size_t len UNUSED, int level UNUSED)
+	     char *buf UNUSED, size_t len UNUSED)
 {
+    /* this is where we implement source-device filtering */
+    if (gpsdata->dev.path[0] && source.device!=NULL && strcmp(source.device, gpsdata->dev.path) != 0)
+	return;
+
     conditionally_log_fix(&gpsdata->fix);
 }
 
@@ -279,46 +294,16 @@ static int socket_mainloop(void)
 
     gpsdata = gps_open(source.server, source.port);
     if (!gpsdata) {
-	char *err_str;
-	switch (errno) {
-	case NL_NOSERVICE:
-	    err_str = "can't get service entry";
-	    break;
-	case NL_NOHOST:
-	    err_str = "can't get host entry";
-	    break;
-	case NL_NOPROTO:
-	    err_str = "can't get protocol entry";
-	    break;
-	case NL_NOSOCK:
-	    err_str = "can't create socket";
-	    break;
-	case NL_NOSOCKOPT:
-	    err_str = "error SETSOCKOPT SO_REUSEADDR";
-	    break;
-	case NL_NOCONNECT:
-	    err_str = "can't connect to host";
-	    break;
-	default:
-	    err_str = "Unknown";
-	    break;
-	}
 	fprintf(stderr,
 		"%s: no gpsd running or network error: %d, %s\n",
-		progname, errno, err_str);
+		progname, errno, gps_errstr(errno));
 	exit(1);
     }
 
-    if (casoc)
-	gps_query(gpsdata, "j1\n");
-    if (source.device != NULL)
-	gps_query(gpsdata, "f=%s\n", source.device);
-
     gps_set_raw_hook(gpsdata, process);
+    gps_stream(gpsdata, WATCH_ENABLE|WATCH_NEWSTYLE, NULL);
 
-    gps_query(gpsdata, "w+x");
-
-    for(;;){
+    for(;;) {
 	int data;
 	struct timeval tv;
 
@@ -329,7 +314,7 @@ static int socket_mainloop(void)
 	tv.tv_sec = 0;
 	data = select(gpsdata->gps_fd + 1, &fds, NULL, NULL, &tv);
 
-	if (data < 0) {
+	if (data == -1) {
 	    (void)fprintf(stderr,"%s\n", strerror(errno));
 	    break;
 	}
@@ -351,7 +336,7 @@ static void usage(void)
 	    "Usage: %s [-V] [-h] [-i timeout] [-j casoc] [server[:port:[device]]]\n",
 	    progname);
     fprintf(stderr,
-	    "\tdefaults to '%s -i 5 -j 0 127.0.0.1:2947'\n",
+	    "\tdefaults to '%s -i 5 -j 0 localhost:2947'\n",
 	    progname);
     exit(1);
 }
@@ -361,8 +346,14 @@ int main (int argc, char** argv)
     int ch;
 
     progname = argv[0];
-    while ((ch = getopt(argc, argv, "hi:j:V")) != -1){
+    while ((ch = getopt(argc, argv, "D:hi:V")) != -1) {
 	switch (ch) {
+	case 'D':
+	    debug = atoi(optarg);
+#ifdef CLIENTDEBUG_ENABLE
+	    gps_enable_debug(debug, stdout);
+#endif /* CLIENTDEBUG_ENABLE */
+	    break;
 	case 'i':		/* set polling interfal */
 	    timeout = (unsigned int)atoi(optarg);
 	    if (timeout < 1)
@@ -371,12 +362,8 @@ int main (int argc, char** argv)
 		fprintf(stderr,
 			"WARNING: track timeout is an hour or more!\n");
 	    break;
-	case 'j':		/* set data smoothing */
-	    casoc = (unsigned int)atoi(optarg);
-	    casoc = casoc ? 1 : 0;
-	    break;
 	case 'V':
-	    (void)fprintf(stderr, "SVN ID: $Id: gpxlogger.c 5326 2009-03-02 23:24:03Z esr $ \n");
+	    (void)fprintf(stderr, "SVN ID: $Id: gpxlogger.c 6920 2010-01-12 19:22:47Z esr $ \n");
 	    exit(0);
 	default:
  	    usage();
@@ -407,7 +394,7 @@ int main (int argc, char** argv)
     print_gpx_header ();
 
 #ifdef DBUS_ENABLE
-    /* To force socket use in the default way just give a '127.0.0.1' arg */  
+    /* To force socket use in the default way just give a 'localhost' arg */
     if (optind < argc)
 	return socket_mainloop();
     else
