@@ -1,4 +1,4 @@
-/* $Id: isgps.c 4916 2009-01-06 16:54:33Z ckuethe $ */
+/* $Id: isgps.c 6566 2009-11-20 03:51:06Z esr $ */
 /*****************************************************************************
 
 This is a decoder for the unnamed protocol described in IS-GPS-200,
@@ -57,12 +57,13 @@ You are not expected to understand any of this.
 *****************************************************************************/
 
 #include <sys/types.h>
+#ifndef S_SPLINT_S
 #include <unistd.h>
+#endif /* S_SPLINT_S */
 #include <stdlib.h>
 #include <string.h>
 #include <stdbool.h>
 
-#include "gpsd_config.h"
 #include "gpsd.h"
 
 #define MAG_SHIFT 6u
@@ -260,7 +261,7 @@ enum isgpsstat_t isgps_decode(struct gps_packet_t *session,
 		     * Guard against a buffer overflow attack.  Just wait for
 		     * the next preamble match and go on from there. 
 		     */
-		    if (session->isgps.bufindex >= (unsigned)maxlen){
+		    if (session->isgps.bufindex >= (unsigned)maxlen) {
 			session->isgps.bufindex = 0;
 			gpsd_report(ISGPS_ERRLEVEL_BASE+1, 
 				    "ISGPS buffer overflowing -- resetting\n");
