@@ -1,21 +1,31 @@
-/* $Id: libgpsd_core.c 5498 2009-03-18 17:37:53Z esr $ */
+/* $Id: libgpsd_core.c 6920 2010-01-12 19:22:47Z esr $ */
 /* libgpsd_core.c -- direct access to GPSes on serial or USB devices. */
+#include <stdlib.h>
+#include "gpsd_config.h"
 #include <sys/time.h>
-#include <sys/ioctl.h>
+#ifdef HAVE_SYS_IOCTL_H
+ #include <sys/ioctl.h>
+#endif /* HAVE_SYS_IOCTL_H */
 #ifndef S_SPLINT_S
-#include <sys/socket.h>
+ #ifdef HAVE_SYS_SOCKET_H
+  #include <sys/socket.h>
+ #else
+  #define AF_UNSPEC 0
+ #endif /* HAVE_SYS_SOCKET_H */
+ #include <unistd.h>
 #endif /* S_SPLINT_S */
 #include <sys/time.h>
-#include <unistd.h>
-#include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
-#include <netdb.h>
+#ifndef S_SPLINT_S
+ #ifdef HAVE_NETDB_H
+  #include <netdb.h>
+ #endif /* HAVE_NETDB_H */
+#endif /* S_SPLINT_S */
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
 
-#include "gpsd_config.h"
 #include "gpsd.h"
 
 #if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
@@ -28,18 +38,11 @@
 int gpsd_switch_driver(struct gps_device_t *session, char* type_name)
 {
     const struct gps_type_t **dp;
+    bool identified = (session->device_type != NULL);
 
     gpsd_report(LOG_PROG, "switch_driver(%s) called...\n", type_name);
-    if (session->device_type != NULL &&
-	strcmp(session->device_type->type_name, type_name) == 0) {
-#ifdef ALLOW_RECONFIGURE
-	gpsd_report(LOG_PROG, "Reconfiguring for %s...\n", session->device_type->type_name);
-	if (session->enable_reconfigure
-		&& session->device_type->configurator != NULL)
-	    session->device_type->configurator(session, 0);
-#endif /* ALLOW_RECONFIGURE */
+    if (identified && strcmp(session->device_type->type_name, type_name) == 0)
 	return 0;
-    }
 
     /*@ -compmempass @*/
     for (dp = gpsd_drivers; *dp; dp++)
@@ -47,16 +50,14 @@ int gpsd_switch_driver(struct gps_device_t *session, char* type_name)
 	    gpsd_report(LOG_PROG, "selecting %s driver...\n", (*dp)->type_name);
 	    gpsd_assert_sync(session);
 	    /*@i@*/session->device_type = *dp;
-	    session->gpsdata.mincycle = session->device_type->min_cycle;
-	    if (!session->context->readonly && session->device_type->probe_subtype != NULL)
-		session->device_type->probe_subtype(session, session->packet.counter = 0);
 #ifdef ALLOW_RECONFIGURE
-	    if (session->enable_reconfigure
-			&& session->device_type->configurator != NULL) {
-		gpsd_report(LOG_PROG, "configuring for %s...\n", session->device_type->type_name);
-		session->device_type->configurator(session, 0);
-	    }
+	    session->gpsdata.dev.mincycle = session->device_type->min_cycle;
 #endif /* ALLOW_RECONFIGURE */
+	    /* reconfiguration might be required */
+	    if (identified && session->device_type->event_hook != NULL)
+		session->device_type->event_hook(session, event_driver_switch);
+	    /* clients should be notified */
+	    session->notify_clients = true;
 	    return 1;
 	}
     gpsd_report(LOG_ERROR, "invalid GPS type \"%s\".\n", type_name);
@@ -70,29 +71,30 @@ void gpsd_init(struct gps_device_t *session, struct gps_context_t *context, char
 {
     /*@ -mayaliasunique @*/
     if (device != NULL)
-	(void)strlcpy(session->gpsdata.gps_device, device, PATH_MAX);
+	(void)strlcpy(session->gpsdata.dev.path, device, sizeof(session->gpsdata.dev.path));
     /*@ -mustfreeonly @*/
     session->device_type = NULL;	/* start by hunting packets */
+    session->observed = 0;
     session->rtcmtime = 0;
+    session->is_serial = false;		/* gpsd_open() setss this */
     /*@ -temptrans @*/
     session->context = context;
     /*@ +temptrans @*/
     /*@ +mayaliasunique @*/
     /*@ +mustfreeonly @*/
     gps_clear_fix(&session->gpsdata.fix);
-    session->gpsdata.set &=~ (FIX_SET | DOP_SET);
-    session->gpsdata.hdop = NAN;
-    session->gpsdata.vdop = NAN;
-    session->gpsdata.pdop = NAN;
-    session->gpsdata.tdop = NAN;
-    session->gpsdata.gdop = NAN;
+    session->gpsdata.set = 0;
+    session->gpsdata.dop.hdop = NAN;
+    session->gpsdata.dop.vdop = NAN;
+    session->gpsdata.dop.pdop = NAN;
+    session->gpsdata.dop.tdop = NAN;
+    session->gpsdata.dop.gdop = NAN;
     session->gpsdata.epe = NAN;
     session->mag_var = NAN;
-    session->gpsdata.cycle = session->gpsdata.mincycle = 1; 
+    session->gpsdata.dev.cycle = session->gpsdata.dev.mincycle = 1;
 
     /* tty-level initialization */
     gpsd_tty_init(session);
-
     /* necessary in case we start reading in the middle of a GPGSV sequence */
     gpsd_zero_satellites(&session->gpsdata);
 
@@ -112,21 +114,18 @@ void gpsd_deactivate(struct gps_device_t *session)
 # endif /* PPS_ENABLE */
 #endif /* NTPSHM_ENABLE */
 #ifdef ALLOW_RECONFIGURE
-    if (session->enable_reconfigure
+    if (!session->context->readonly
 	&& session->device_type != NULL
-	&& session->device_type->revert != NULL) {
-	session->device_type->revert(session);
-	session->enable_reconfigure = false;
+	&& session->device_type->event_hook != NULL) {
+	session->device_type->event_hook(session, event_deactivate);
     }
     if (session->device_type!=NULL) {
 	if (session->back_to_nmea && session->device_type->mode_switcher!=NULL)
 	    session->device_type->mode_switcher(session, 0);
-	if (session->device_type->wrapup!=NULL)
-	    session->device_type->wrapup(session);
     }
 #endif /* ALLOW_RECONFIGURE */
     gpsd_report(LOG_INF, "closing GPS=%s (%d)\n",
-		session->gpsdata.gps_device, session->gpsdata.gps_fd);
+		session->gpsdata.dev.path, session->gpsdata.gps_fd);
     (void)gpsd_close(session);
 }
 
@@ -137,39 +136,72 @@ static /*@null@*/void *gpsd_ppsmonitor(void *arg)
     int cycle,duration, state = 0, laststate = -1, unchanged = 0;
     struct timeval tv;
     struct timeval pulse[2] = {{0,0},{0,0}};
-    int pps_device = TIOCM_CAR;
 
 #if defined(PPS_ON_CTS)
-    pps_device = TIOCM_CTS;
+    int pps_device = TIOCM_CTS;
+    #define pps_device_str "CTS"
+#else
+    int pps_device = TIOCM_CAR;
+    #define pps_device_str "DCD"
 #endif
 
-    gpsd_report(LOG_PROG, "Create Thread gpsd_ppsmonitor\n");
+    gpsd_report(LOG_PROG, "PPS Create Thread gpsd_ppsmonitor\n");
 
     /* wait for status change on the device's carrier-detect line */
     while (ioctl(session->gpsdata.gps_fd, TIOCMIWAIT, pps_device) == 0) {
+	int ok = 0;
+	char *log = NULL;
+
 	(void)gettimeofday(&tv,NULL);
+
+	ok = 0;
+	log = NULL;
+
 	/*@ +ignoresigns */
 	if (ioctl(session->gpsdata.gps_fd, TIOCMGET, &state) != 0)
 	    break;
 	/*@ -ignoresigns */
 
 	state = (int)((state & pps_device) != 0);
+	/*@ +boolint @*/
+#define timediff(x, y)	(int)((x.tv_sec-y.tv_sec)*1000000+x.tv_usec-y.tv_usec)
+	cycle = timediff(tv, pulse[state]);
+	duration = timediff(tv, pulse[(int)(state == 0)]);
+#undef timediff
+	/*@ -boolint @*/
 
 	if (state == laststate) {
-	    if (++unchanged == 10) {
-		gpsd_report(LOG_WARN, "TIOCMIWAIT returns unchanged state, ppsmonitor terminates\n");
-		break;
+	    /* some pulses may be so short that state never changes */
+	    if ( 999000 < cycle && 1001000 > cycle ) {
+		duration = 0;
+		unchanged = 0;
+		gpsd_report(LOG_RAW,
+			"PPS pps-detect (%s) on %s invisible pulse\n",
+			pps_device_str, session->gpsdata.dev.path);
+	    } else if (++unchanged == 10) {
+		unchanged = 1;
+		gpsd_report(LOG_WARN,
+	"PPS TIOCMIWAIT returns unchanged state, ppsmonitor sleeps 10\n");
+		(void)sleep(10);
 	    }
 	} else {
-	    gpsd_report(LOG_RAW, "pps-detect (%s) on %s changed to %d\n",
-			((pps_device==TIOCM_CAR) ? "DCD" : "CTS"),
-			  session->gpsdata.gps_device, state);
+	    gpsd_report(LOG_RAW, "PPS pps-detect (%s) on %s changed to %d\n",
+			  pps_device_str,
+			  session->gpsdata.dev.path, state);
 	    laststate = state;
 	    unchanged = 0;
 	}
+	pulse[state] = tv;
+	if ( unchanged ) {
+	    // strange, try again
+	    continue;
+	}
+	gpsd_report(LOG_INF, "PPS cycle: %d, duration: %d @ %lu.%06lu\n",
+	    cycle, duration,
+	    (unsigned long)tv.tv_sec, (unsigned long)tv.tv_usec);
 
 	/*@ +boolint @*/
-	if ( session->context->fixcnt > 3 ) {
+	if ( 3 < session->context->fixcnt ) {
 	    /* Garmin doc says PPS is valid after four good fixes. */
 	    /*
 	     * The PPS pulse is normally a short pulse with a frequency of
@@ -184,58 +216,91 @@ static /*@null@*/void *gpsd_ppsmonitor(void *arg)
 	     * Some GPS instead output a square wave that is 0.5 Hz and each
 	     * edge denotes the start of a second.
 	     *
-	     * A few stupid GPS, like the Furuno GPSClock, output a 1.0 Hz 
+	     * Some GPS, like the Globalsat MR-350P, output a 1uS pulse.
+	     * The pulse is so short that TIOCMIWAIT sees a state change
+	     * but by the time TIOCMGET is called the pulse is gone.
+	     *
+	     * A few stupid GPS, like the Furuno GPSClock, output a 1.0 Hz
 	     * square wave where the leading edge is the start of a second
 	     *
-	     * 5Hz GPS (Garmin 18-5Hz) pulses at 5Hz. Set the pulse length to 
+	     * 5Hz GPS (Garmin 18-5Hz) pulses at 5Hz. Set the pulse length to
 	     * 40ms which gives a 160ms pulse before going high.
 	     *
 	     */
-#define timediff(x, y)	(int)((x.tv_sec-y.tv_sec)*1000000+x.tv_usec-y.tv_usec)
-	    cycle = timediff(tv, pulse[state]);
-	    duration = timediff(tv, pulse[state == 0]);
-#undef timediff
 
-	    if (cycle > 199000 && cycle < 201000 ) {
+	    if (199000 > cycle) {
+		// too short to even be a 5Hz pulse
+		log = "Too short for 5Hz\n";
+	    } else if (201000 > cycle) {
 		/* 5Hz cycle */
 		/* looks like 5hz PPS pulse */
-		if (duration > 45000)
-		    (void)ntpshm_pps(session, &tv);
-		gpsd_report(LOG_RAW, "5Hz PPS pulse. cycle: %d, duration: %d\n",
-			cycle, duration);
-	    } else if (cycle > 999000 && cycle < 1001000 ) {
+		if (100000 > duration) {
+		    /* BUG: how does the code know to tell ntpd
+		     * which 1/5 of a second to use?? */
+		    ok = 1;
+		    log = "5Hz PPS pulse\n";
+		}
+	    } else if (999000 > cycle) {
+		    log = "Too long for 5Hz, too short for 1Hz\n";
+	    } else if (1001000 > cycle) {
 		/* looks like PPS pulse or square wave */
-		if (duration > 499000 && duration < 501000
-#ifdef GPSCLOCK_ENABLE
-		  && session->driver.nmea.ignore_trailing_edge
+
+#if 0
+/* huh? */
+#if defined(NMEA_ENABLE) && defined(GPSCLOCK_ENABLE)
+		 && session->driver.nmea.ignore_trailing_edge
 #endif /* GPSCLOCK_ENABLE */
-		  ) {
+#endif
+
+		if (0 == duration) {
+		    ok = 1;
+		    log = "PPS invisible pulse\n";
+		} else if (499000 > duration) {
+		    /* end of the short "half" of the cycle */
+		    /* aka the trailing edge */
+		    log = "PPS 1Hz trailing edge\n";
+		} else if (501000 > duration) {
 		    /* looks like 1.0 Hz square wave, ignore trailing edge */
 		    if (state == 1) {
-			 (void)ntpshm_pps(session, &tv);
+			ok = 1;
+			log = "PPS square\n";
 		    }
 		} else {
-		    /* looks like PPS pulse */
-		    (void)ntpshm_pps(session, &tv);
+		    /* end of the long "half" of the cycle */
+		    /* aka the leading edge */
+		    ok = 1;
+		    log = "PPS 1Hz leading edge\n";
 		}
-		gpsd_report(LOG_RAW, "PPS pulse. cycle: %d, duration: %d\n",
-			cycle, duration);
-
-	    } else if (cycle > 1999000 && cycle < 2001000) {
+	    } else if (1999000 > cycle) {
+		log = "Too long for 1Hz, too short for 2Hz\n";
+	    } else if (2001000 > cycle) {
 		/* looks like 0.5 Hz square wave */
-		(void)ntpshm_pps(session, &tv);
-		gpsd_report(LOG_RAW, "PPS square wave. cycle: %d, duration: %d\n",
-			cycle, duration);
+                if (999000 > duration) {
+                    log = "PPS 0.5 Hz square too short duration\n";
+                } else if (1001000 > duration) {
+                    ok = 1;
+                    log = "PPS 0.5 Hz square wave\n";
+                } else {
+                    log = "PPS 0.5 Hz square too long duration\n";
+                }
 	    } else {
-		gpsd_report(LOG_INF, "PPS pulse rejected.  cycle: %d, duration: %d\n",
-			cycle, duration);
+		log = "Too long for 0.5Hz\n";
 	    }
 	} else {
-		gpsd_report(LOG_INF, "PPS pulse rejected. No fix.\n");
+	    /* not a good fix, but a test for an otherwise good PPS
+	     * would go here */
+	    log = "PPS no fix.\n";
 	}
 	/*@ -boolint @*/
+	if ( NULL != log ) {
+	    gpsd_report(LOG_RAW, "%s", log);
+	}
+	if ( 0 != ok ) {
+	    (void)ntpshm_pps(session, &tv);
+	} else {
+	    gpsd_report(LOG_INF, "PPS pulse rejected\n");
+	}
 
-	pulse[state] = tv;
     }
 
     return NULL;
@@ -243,10 +308,38 @@ static /*@null@*/void *gpsd_ppsmonitor(void *arg)
 #endif /* PPS_ENABLE */
 
 /*@ -branchstate @*/
-int gpsd_activate(struct gps_device_t *session, bool reconfigurable)
+int gpsd_activate(struct gps_device_t *session)
 /* acquire a connection to the GPS device */
 {
-    if (gpsd_open(session) < 0)
+    /* special case: source may be a URI to a remote GNSS or DGPS service */
+    if (netgnss_uri_check(session->gpsdata.dev.path))
+	session->gpsdata.gps_fd = netgnss_uri_open(session->context, 
+						   session->gpsdata.dev.path);
+    /* otherwise, could be an AIS data feed */
+    else if (strncmp(session->gpsdata.dev.path, "ais://", 6) == 0) {
+	char server[GPS_PATH_MAX], *port;
+	socket_t dsock;
+	(void)strlcpy(server, session->gpsdata.dev.path+6, sizeof(server));
+	session->gpsdata.gps_fd = -1;
+	port = strchr(server, ':');
+	if (port == NULL) {
+	    gpsd_report(LOG_ERROR, "Missing colon in AIS feed spec.\n");
+	    return -1;
+	}
+	*port++ = '\0';
+	gpsd_report(LOG_INF, "opening AIS feed at %s, port %s.\n", server,port);
+	if ((dsock = netlib_connectsock(AF_UNSPEC, server, port, "tcp")) < 0) {
+	    gpsd_report(LOG_ERROR, "AIS device open error %s.\n", 
+			netlib_errstr(dsock));
+	    return -1;
+	}
+	session->gpsdata.gps_fd = dsock;
+    }
+    /* otherwise, ordinary serial device */
+    else 
+	session->gpsdata.gps_fd = gpsd_open(session);
+
+    if (session->gpsdata.gps_fd < 0)
 	return -1;
     else {
 #ifdef NON_NMEA_ENABLE
@@ -261,13 +354,10 @@ int gpsd_activate(struct gps_device_t *session, bool reconfigurable)
 		gpsd_assert_sync(session);
 		goto foundit;
 	    }
- 	}
+	}
 	/*@ +mustfreeonly @*/
 	gpsd_report(LOG_PROG, "no probe matched...\n");
     foundit:
-#ifdef ALLOW_RECONFIGURE
-	session->enable_reconfigure = reconfigurable;
-#endif /* ALLOW_RECONFIGURE */
 #endif /* NON_NMEA_ENABLE */
 	session->gpsdata.online = timestamp();
 #ifdef SIRF_ENABLE
@@ -275,28 +365,28 @@ int gpsd_activate(struct gps_device_t *session, bool reconfigurable)
 #endif /* SIRF_ENABLE */
 	session->packet.char_counter = 0;
 	session->packet.retry_counter = 0;
-	gpsd_report(LOG_INF, "gpsd_activate(%d): opened GPS (%d)\n", reconfigurable, session->gpsdata.gps_fd);
+	gpsd_report(LOG_INF,
+		    "gpsd_activate(): opened GPS (fd %d)\n",
+		    session->gpsdata.gps_fd);
 	// session->gpsdata.online = 0;
 	session->gpsdata.fix.mode = MODE_NOT_SEEN;
 	session->gpsdata.status = STATUS_NO_FIX;
 	session->gpsdata.fix.track = NAN;
 	session->gpsdata.separation = NAN;
 	session->mag_var = NAN;
+	session->releasetime = 0;
 
-	/* clear driver subtype field and private data union */
-	session->subtype[0] = '\0';
+	/* clear the private data union */
 	memset(&session->driver, '\0', sizeof(session->driver));
-	/* if we know the device type, probe for subtype and configure it */
-	if (session->device_type != NULL) {
-	    if (!session->context->readonly && session->device_type->probe_subtype !=NULL)
-		session->device_type->probe_subtype(session, session->packet.counter = 0);
-#ifdef ALLOW_RECONFIGURE
-	    if (reconfigurable) {
-		if (session->device_type->configurator != NULL)
-		    session->device_type->configurator(session, session->packet.counter);
-	    }
-#endif /* ALLOW_RECONFIGURE */
-	}
+	/*
+	 * We might know the device's type, but we shoudn't assume it has
+	 * retained its settings.  A revert hook might well have undone
+	 * them on the previous close.  Fire a reactivate event so drivers
+	 * can do something about this if they choose.
+	 */
+	if (session->device_type != NULL
+	    && session->device_type->event_hook != NULL)
+	    session->device_type->event_hook(session, event_reactivate);
     }
 
     return session->gpsdata.gps_fd;
@@ -314,15 +404,22 @@ void ntpd_link_activate(struct gps_device_t *session)
     if (session->context->enable_ntpshm)
 	session->shmindex = ntpshm_alloc(session->context);
 
+    if ( 0 > session->shmindex ) {
+	gpsd_report(LOG_INF, "NTPD ntpshm_alloc() failed\n");
 #if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
-    /* If we also have the 1pps capability, allocate a shared-memory segment for
-     * the 1pps time data and launch a thread to capture the 1pps transitions
-     */
-    if (session->shmindex >= 0 && session->context->shmTimePPS)
-	if ((session->shmTimeP = ntpshm_alloc(session->context)) >= 0)
+    } else if ( session->context->shmTimePPS) {
+	/* We also have the 1pps capability, allocate a shared-memory segment
+	 * for the 1pps time data and launch a thread to capture the 1pps
+	 * transitions
+	 */
+	if ((session->shmTimeP = ntpshm_alloc(session->context)) >= 0) {
 	    /*@i1@*/(void)pthread_create(&pt,NULL,gpsd_ppsmonitor,(void *)session);
+	} else {
+	    gpsd_report(LOG_INF, "NTPD ntpshm_alloc(1) failed\n");
+	}
 
 #endif /* defined(PPS_ENABLE) && defined(TIOCMIWAIT) */
+    }
 #endif /* NTPSHM_ENABLE */
 }
 
@@ -340,229 +437,6 @@ char /*@observer@*/ *gpsd_id(/*@in@*/struct gps_device_t *session)
     }
     return(buf);
 }
-
-#if defined(BINARY_ENABLE) || defined(RTCM104V2_ENABLE) || defined(NTRIP_ENABLE)
-/*
- * Support for generic binary drivers.  These functions dump NMEA for passing
- * to the client in raw mode.  They assume that (a) the public gps.h structure 
- * members are in a valid state, (b) that the private members hours, minutes, 
- * and seconds have also been filled in, (c) that if the private member
- * mag_var is not NAN it is a magnetic variation in degrees that should be
- * passed on, and (d) if the private member separation does not have the
- * value NAN, it is a valid WGS84 geoidal separation in meters for the fix.
- */
-
-static double degtodm(double angle)
-/* decimal degrees to GPS-style, degrees first followed by minutes */
-{
-    double fraction, integer;
-    fraction = modf(angle, &integer);
-    return floor(angle) * 100 + fraction * 60;
-}
-
-/*@ -mustdefine @*/
-void gpsd_position_fix_dump(struct gps_device_t *session,
-			    /*@out@*/char bufp[], size_t len)
-{
-    struct tm tm;
-    time_t intfixtime;
-
-    intfixtime = (time_t)session->gpsdata.fix.time;
-    (void)gmtime_r(&intfixtime, &tm);
-    if (session->gpsdata.fix.mode > 1) {
-	(void)snprintf(bufp, len,
-		"$GPGGA,%02d%02d%02d,%09.4f,%c,%010.4f,%c,%d,%02d,",
-		tm.tm_hour,
-		tm.tm_min,
-		tm.tm_sec,
-		degtodm(fabs(session->gpsdata.fix.latitude)),
-		((session->gpsdata.fix.latitude > 0) ? 'N' : 'S'),
-		degtodm(fabs(session->gpsdata.fix.longitude)),
-		((session->gpsdata.fix.longitude > 0) ? 'E' : 'W'),
-		session->gpsdata.status,
-		session->gpsdata.satellites_used);
-	if (isnan(session->gpsdata.hdop))
-	    (void)strlcat(bufp, ",", len);
-	else
-	    (void)snprintf(bufp+strlen(bufp), len-strlen(bufp),
-			   "%.2f,",session->gpsdata.hdop);
-	if (isnan(session->gpsdata.fix.altitude))
-	    (void)strlcat(bufp, ",", len);
-	else
-	    (void)snprintf(bufp+strlen(bufp), len-strlen(bufp),
-			   "%.2f,M,", session->gpsdata.fix.altitude);
-	if (isnan(session->gpsdata.separation))
-	    (void)strlcat(bufp, ",", len);
-	else
-	    (void)snprintf(bufp+strlen(bufp), len-strlen(bufp),
-			   "%.3f,M,", session->gpsdata.separation);
-	if (isnan(session->mag_var))
-	    (void)strlcat(bufp, ",", len);
-	else {
-	    (void)snprintf(bufp+strlen(bufp),
-			   len-strlen(bufp),
-			   "%3.2f,", fabs(session->mag_var));
-	    (void)strlcat(bufp, (session->mag_var > 0) ? "E": "W", len);
-	}
-	nmea_add_checksum(bufp);
-    }
-}
-/*@ +mustdefine @*/
-
-static void gpsd_transit_fix_dump(struct gps_device_t *session,
-				  char bufp[], size_t len)
-{
-    struct tm tm;
-    time_t intfixtime;
-
-    tm.tm_mday = tm.tm_mon = tm.tm_year = tm.tm_hour = tm.tm_min = tm.tm_sec = 0;
-    if (isnan(session->gpsdata.fix.time)==0) {
-	intfixtime = (time_t)session->gpsdata.fix.time;
-	(void)gmtime_r(&intfixtime, &tm);
-	tm.tm_mon++;
-	tm.tm_year %= 100;
-    }
-#define ZEROIZE(x)	(isnan(x)!=0 ? 0.0 : x)
-    /*@ -usedef @*/
-    (void)snprintf(bufp, len,
-	    "$GPRMC,%02d%02d%02d,%c,%09.4f,%c,%010.4f,%c,%.4f,%.3f,%02d%02d%02d,,",
-	    tm.tm_hour,
-	    tm.tm_min,
-	    tm.tm_sec,
-	    session->gpsdata.status ? 'A' : 'V',
-	    degtodm(fabs(session->gpsdata.fix.latitude)),
-	    ((session->gpsdata.fix.latitude > 0) ? 'N' : 'S'),
-	    degtodm(fabs(session->gpsdata.fix.longitude)),
-	    ((session->gpsdata.fix.longitude > 0) ? 'E' : 'W'),
-	    ZEROIZE(session->gpsdata.fix.speed * MPS_TO_KNOTS),
-	    ZEROIZE(session->gpsdata.fix.track),
-	    tm.tm_mday,
-	    tm.tm_mon,
-	    tm.tm_year);
-    /*@ +usedef @*/
-#undef ZEROIZE
-    nmea_add_checksum(bufp);
-}
-
-static void gpsd_binary_fix_dump(struct gps_device_t *session,
-				 char bufp[], size_t len)
-{
-    gpsd_position_fix_dump(session, bufp, len);
-    gpsd_transit_fix_dump(session, bufp + strlen(bufp), len - strlen(bufp));
-}
-
-static void gpsd_binary_satellite_dump(struct gps_device_t *session,
-				char bufp[], size_t len)
-{
-    int i;
-    char *bufp2 = bufp;
-    bufp[0] = '\0';
-
-    for( i = 0 ; i < session->gpsdata.satellites; i++ ) {
-	if (i % 4 == 0) {
-	    bufp += strlen(bufp);
-	    bufp2 = bufp;
-	    len -= snprintf(bufp, len,
-		    "$GPGSV,%d,%d,%02d",
-		    ((session->gpsdata.satellites-1) / 4) + 1,
-		    (i / 4) + 1,
-		    session->gpsdata.satellites);
-	}
-	bufp += strlen(bufp);
-	if (i < session->gpsdata.satellites)
-	    len -= snprintf(bufp, len,
-		    ",%02d,%02d,%03d,%02d",
-		    session->gpsdata.PRN[i],
-		    session->gpsdata.elevation[i],
-		    session->gpsdata.azimuth[i],
-		    session->gpsdata.ss[i]);
-	if (i % 4 == 3 || i == session->gpsdata.satellites-1) {
-	    nmea_add_checksum(bufp2);
-	    len -= 5;
-	}
-    }
-
-#ifdef ZODIAC_ENABLE
-    if (session->packet.type == ZODIAC_PACKET && session->driver.zodiac.Zs[0] != 0) {
-	bufp += strlen(bufp);
-	bufp2 = bufp;
-	(void)strlcpy(bufp, "$PRWIZCH", len);
-	for (i = 0; i < ZODIAC_CHANNELS; i++) {
-	    len -= snprintf(bufp+strlen(bufp), len,
-			  ",%02u,%X",
-			    session->driver.zodiac.Zs[i],
-			    session->driver.zodiac.Zv[i] & 0x0f);
-	}
-	nmea_add_checksum(bufp2);
-    }
-#endif /* ZODIAC_ENABLE */
-}
-
-static void gpsd_binary_quality_dump(struct gps_device_t *session,
-			      char bufp[], size_t len)
-{
-    int	i, j;
-    char *bufp2 = bufp;
-
-    (void)snprintf(bufp, len-strlen(bufp),
-		   "$GPGSA,%c,%d,", 'A', session->gpsdata.fix.mode);
-    j = 0;
-    for (i = 0; i < session->device_type->channels; i++) {
-	if (session->gpsdata.used[i]) {
-	    bufp += strlen(bufp);
-	    (void)snprintf(bufp, len-strlen(bufp),
-			   "%02d,", session->gpsdata.used[i]);
-	    j++;
-	}
-    }
-    for (i = j; i < session->device_type->channels; i++) {
-	bufp += strlen(bufp);
-	(void)strlcpy(bufp, ",", len);
-    }
-    bufp += strlen(bufp);
-#define ZEROIZE(x)	(isnan(x)!=0 ? 0.0 : x)
-    if (session->gpsdata.fix.mode == MODE_NO_FIX)
-	(void)strlcat(bufp, ",,,", len);
-    else
-	(void)snprintf(bufp, len-strlen(bufp),
-		       "%.1f,%.1f,%.1f*",
-		       ZEROIZE(session->gpsdata.pdop),
-		       ZEROIZE(session->gpsdata.hdop),
-		       ZEROIZE(session->gpsdata.vdop));
-    nmea_add_checksum(bufp2);
-    bufp += strlen(bufp);
-    if (finite(session->gpsdata.fix.eph)
-	|| finite(session->gpsdata.fix.epv)
-	|| finite(session->gpsdata.epe)) {
-	/*
-	 * Output PGRME only if realistic.  Note: we're converting back to
-	 * our guess about Garmin's confidence units here, make sure this
-	 * stays consistent with the in-conversion in nmea_parse.c!
-	 */
-	(void)snprintf(bufp, len-strlen(bufp),
-	    "$PGRME,%.2f,M,%.2f,M,%.2f,M",
-	    ZEROIZE(session->gpsdata.fix.eph * (CEP50_SIGMA/GPSD_CONFIDENCE)),
-	    ZEROIZE(session->gpsdata.fix.epv * (CEP50_SIGMA/GPSD_CONFIDENCE)),
-	    ZEROIZE(session->gpsdata.epe * (CEP50_SIGMA/GPSD_CONFIDENCE)));
-	nmea_add_checksum(bufp);
-     }
-#undef ZEROIZE
-}
-
-static void gpsd_binary_dump(struct gps_device_t *session,
-			      char bufp[], size_t len)
-{
-    if ((session->gpsdata.set & LATLON_SET) != 0)
-	gpsd_binary_fix_dump(session, bufp+strlen(bufp), len-strlen(bufp));
-    if ((session->gpsdata.set & (DOP_SET | ERR_SET)) != 0)
-	gpsd_binary_quality_dump(session, bufp+strlen(bufp), len-strlen(bufp));
-    if ((session->gpsdata.set & SATELLITE_SET) != 0)
-	gpsd_binary_satellite_dump(session,bufp+strlen(bufp),len-strlen(bufp));
-}
-
-#endif /* BINARY_ENABLE */
-
-
 void gpsd_error_model(struct gps_device_t *session,
 		      struct gps_fix_t *fix, struct gps_fix_t *oldfix)
 /* compute errors and derived quantities */
@@ -593,41 +467,59 @@ void gpsd_error_model(struct gps_device_t *session,
     v_uere = (session->gpsdata.status == STATUS_DGPS_FIX ? V_UERE_WITH_DGPS : V_UERE_NO_DGPS);
     p_uere = (session->gpsdata.status == STATUS_DGPS_FIX ? P_UERE_WITH_DGPS : P_UERE_NO_DGPS);
 
+
+    /*
+     * OK, this is not an error computation, but
+     * we're at the right place in the architrcture for it.
+     * Compute climb/sink in the simplest possible way.
+     * FIXME: Someday we should compute speed here too.
+     */
+    if (fix->mode>=MODE_3D && oldfix->mode>=MODE_3D && isnan(fix->climb)!=0) {
+	if (fix->time == oldfix->time)
+	    fix->climb = 0;
+	else if (isnan(fix->altitude)==0 && isnan(oldfix->altitude)==0) {
+	    fix->climb = (fix->altitude-oldfix->altitude)/(fix->time-oldfix->time);
+	}
+    }
+
     /*
      * Field reports match the theoretical prediction that
      * expected time error should be half the resolution of
      * the GPS clock, so we put the bound of the error
      * in as a constant pending getting it from each driver.
      */
-    if (isnan(fix->ept)!=0)
+    if (isnan(fix->time)==0 && isnan(fix->ept)!=0)
 	fix->ept = 0.005;
     /* Other error computations depend on having a valid fix */
     if (fix->mode >= MODE_2D) {
-	if (isnan(fix->eph)!=0 && finite(session->gpsdata.hdop)!=0)
-		fix->eph = session->gpsdata.hdop * h_uere;
+	if (isnan(fix->epx)!=0 && finite(session->gpsdata.dop.hdop)!=0)
+		fix->epx = session->gpsdata.dop.xdop * h_uere;
+
+	if (isnan(fix->epy)!=0 && finite(session->gpsdata.dop.hdop)!=0)
+		fix->epy = session->gpsdata.dop.ydop * h_uere;
 
 	if ((fix->mode >= MODE_3D)
-		&& isnan(fix->epv)!=0 && finite(session->gpsdata.vdop)!=0)
-	    fix->epv = session->gpsdata.vdop * v_uere;
+		&& isnan(fix->epv)!=0 && finite(session->gpsdata.dop.vdop)!=0)
+	    fix->epv = session->gpsdata.dop.vdop * v_uere;
 
-	if (isnan(session->gpsdata.epe)!=0 && finite(session->gpsdata.pdop)!=0)
-	    session->gpsdata.epe = session->gpsdata.pdop * p_uere;
+	if (isnan(session->gpsdata.epe)!=0 && finite(session->gpsdata.dop.pdop)!=0)
+	    session->gpsdata.epe = session->gpsdata.dop.pdop * p_uere;
 	else
 	    session->gpsdata.epe = NAN;
 
 	/*
-	 * If we have a current fix and an old fix, and the packet handler 
-	 * didn't set the speed error and climb error members itself, 
+	 * If we have a current fix and an old fix, and the packet handler
+	 * didn't set the speed error and climb error members itself,
 	 * try to compute them now.
 	 */
 	if (isnan(fix->eps)!=0)
 	{
 	    if (oldfix->mode > MODE_NO_FIX && fix->mode > MODE_NO_FIX
-			&& isnan(oldfix->eph)==0 && isnan(oldfix->eph)==0
+			&& isnan(oldfix->epx)==0 && isnan(oldfix->epy)==0
 			&& isnan(oldfix->time)==0 && isnan(oldfix->time)==0
 			&& fix->time > oldfix->time) {
 		double t = fix->time-oldfix->time;
-		double e = oldfix->eph + fix->eph;
+		double e = EMIX(oldfix->epx,oldfix->epy) + EMIX(fix->epx,fix->epy);
 		fix->eps = e/t;
 	    } else
 		fix->eps = NAN;
@@ -661,8 +553,8 @@ void gpsd_error_model(struct gps_device_t *session,
 		double adj = earth_distance(
 		    oldfix->latitude, oldfix->longitude,
 		    fix->latitude, fix->longitude);
-		if (isnan(adj)==0 && adj > fix->eph) {
-		    double opp = fix->eph;
+		if (isnan(adj)==0 && adj > EMIX(fix->epx, fix->epy)) {
+		    double opp = EMIX(fix->epx, fix->epy);
 		    double hyp = sqrt(adj*adj + opp*opp);
 		    fix->epd = RAD_2_DEG * 2 * asin(opp / hyp);
 		}
@@ -681,28 +573,35 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 /* update the stuff in the scoreboard structure */
 {
     ssize_t newlen;
+    bool first_sync = false;
 
     gps_clear_fix(&session->gpsdata.fix);
 
-    if (session->packet.inbuflen==0)
-	session->gpsdata.d_xmit_time = timestamp();
+#ifdef TIMING_ENABLE
+    if (session->packet.outbuflen == 0)
+	session->d_xmit_time = timestamp();
+#endif /* TIMING_ENABLE */
+
+    if (session->packet.type >= COMMENT_PACKET)
+	/*@i2@*/session->observed |= PACKET_TYPEMASK(session->packet.type);
 
     /* can we get a full packet from the device? */
     if (session->device_type) {
 	newlen = session->device_type->get_packet(session);
-	session->gpsdata.d_xmit_time = timestamp();
-	if (session->packet.outbuflen>0 && !session->context->readonly && session->device_type->probe_subtype!=NULL)
-	    session->device_type->probe_subtype(session, ++session->packet.counter);
+	gpsd_report(LOG_RAW,
+		    "%s is known to be %s\n",
+		    session->gpsdata.dev.path,
+		    session->device_type->type_name);
     } else {
 	const struct gps_type_t **dp;
 
 	newlen = generic_get(session);
-	session->gpsdata.d_xmit_time = timestamp();
 	gpsd_report(LOG_RAW,
 		    "packet sniff on %s finds type %d\n",
-		    session->gpsdata.gps_device,
+		    session->gpsdata.dev.path,
 		    session->packet.type);
-	if (session->packet.type >= COMMENT_PACKET) {
+	if (session->packet.type > COMMENT_PACKET) {
+	    first_sync = (session->device_type == NULL);
 	    for (dp = gpsd_drivers; *dp; dp++)
 		if (session->packet.type == (*dp)->packet_type) {
 		    (void)gpsd_switch_driver(session, (*dp)->type_name);
@@ -713,47 +612,68 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
     }
 
     /* update the scoreboard structure from the GPS */
-    gpsd_report(LOG_RAW+2, "GPS sent %zd new characters\n", newlen);
-    if (newlen == -1)	{		/* read error */
+    gpsd_report(LOG_RAW+2, "%s sent %zd new characters\n",
+		session->gpsdata.dev.path, newlen);
+   if (newlen == -1)	{		/* read error */
 	gpsd_report(LOG_INF, "GPS on %s is offline (%lf sec since data)\n",
-		    session->gpsdata.gps_device,
+		    session->gpsdata.dev.path,
 		    timestamp() - session->gpsdata.online);
 	session->gpsdata.online = 0;
 	return 0;
     } else if (newlen == 0) {		/* no new data */
-	if (session->device_type != NULL && timestamp()>session->gpsdata.online+session->gpsdata.
-cycle+1){
-	gpsd_report(LOG_INF, "GPS on %s is offline (%lf sec since data)\n",
-		    session->gpsdata.gps_device,
+	if (session->device_type != NULL && timestamp()>session->gpsdata.online+session->gpsdata.dev.cycle+1) {
+	    gpsd_report(LOG_INF, "GPS on %s is offline (%lf sec since data)\n",
+		    session->gpsdata.dev.path,
 		    timestamp() - session->gpsdata.online);
-	session->gpsdata.online = 0;
+	    session->gpsdata.online = 0;
 	    return 0;
 	} else
 	    return ONLINE_SET;
     } else if (session->packet.outbuflen == 0) {   /* got new data, but no packet */
 	gpsd_report(LOG_RAW+3, "New data on %s, not yet a packet\n",
-			    session->gpsdata.gps_device);
+			    session->gpsdata.dev.path);
 	return ONLINE_SET;
-    } else {
-	gps_mask_t received, dopmask = 0;
+    } else {				/* we have recognized a packet */
+	gps_mask_t received = PACKET_SET, dopmask = 0;
 	session->gpsdata.online = timestamp();
 
 	gpsd_report(LOG_RAW+3, "Accepted packet on %s.\n",
-			    session->gpsdata.gps_device);
-	/*@ -nullstate @*/
-	if (session->gpsdata.raw_hook)
-	    session->gpsdata.raw_hook(&session->gpsdata,
-				      (char *)session->packet.outbuffer,
-				      (size_t)session->packet.outbuflen, 2);
-	/*@ -nullstate @*/
-	session->gpsdata.sentence_length = session->packet.outbuflen;
-	session->gpsdata.d_recv_time = timestamp();
+			    session->gpsdata.dev.path);
+
+#ifdef TIMING_ENABLE
+	session->d_recv_time = timestamp();
+#endif /* TIMING_ENABLE */
+
+	/* track the packet count since achieving sync on the device */
+	if (first_sync) {
+	    /* fire the identified hook */
+	    if (session->device_type != NULL && session->device_type->event_hook != NULL)
+		session->device_type->event_hook(session, event_identified);
+	    session->packet.counter = 0;
+	} else
+	    session->packet.counter++;
+
+	/* fire the configure hook */
+	if (session->device_type != NULL && session->device_type->event_hook != NULL)
+	    session->device_type->event_hook(session, event_configure);
+
+	/*
+	 * If this is the first time we've achieved sync on this
+	 * device, or the the driver type has changed for any other
+	 * reason, that's a significant event that the caller needs to
+	 * know about.  Using DEVICE_SET this way is a bit shaky but
+	 * we're short of bits in the flag mask (client library uses
+	 * it differently).
+	 */
+	if (first_sync || session->notify_clients) {
+	    session->notify_clients = false;
+	    received |= DEVICE_SET;
+	}
 
 	/* Get data from current packet into the fix structure */
-	received = 0;
 	if (session->packet.type != COMMENT_PACKET)
 	    if (session->device_type != NULL && session->device_type->parse_packet!=NULL)
-		received = session->device_type->parse_packet(session);
+		received |= session->device_type->parse_packet(session);
 
 	/*
 	 * Compute fix-quality data from the satellite positions.
@@ -761,14 +681,14 @@ cycle+1){
 	 */
 	if (session->gpsdata.fix.mode > MODE_NO_FIX
 		    && (session->gpsdata.set & SATELLITE_SET) != 0
-		    && session->gpsdata.satellites > 0) {
-	    dopmask = dop(&session->gpsdata);
+		    && session->gpsdata.satellites_visible > 0) {
+	    dopmask = fill_dop(&session->gpsdata, &session->gpsdata.dop);
 	    session->gpsdata.epe = NAN;
 	}
 	session->gpsdata.set = ONLINE_SET | dopmask | received;
 
 	/*
-	 * Count good fixes. We used to check 
+	 * Count good fixes. We used to check
 	 *	session->gpsdata.status > STATUS_NO_FIX
 	 * here, but that wasn't quite right.  That tells us whether
 	 * we think we have a valid fix for the current cycle, but remains
@@ -782,44 +702,22 @@ cycle+1){
 	if ((session->gpsdata.set & LATLON_SET )!=0 && session->gpsdata.status > STATUS_NO_FIX)
 	    session->context->fixcnt++;
 
-	session->gpsdata.d_decode_time = timestamp();
+#ifdef TIMING_ENABLE
+	session->d_decode_time = timestamp();
+#endif /* TIMING_ENABLE */
 
-	/* also copy the sentence up to clients in raw mode */
-	if (session->packet.type == NMEA_PACKET) {
-	    if (session->gpsdata.raw_hook)
-		session->gpsdata.raw_hook(&session->gpsdata,
-					  (char *)session->packet.outbuffer,
-					  strlen((char *)session->packet.outbuffer),
-					  1);
-	} else {
-	    char buf2[MAX_PACKET_LENGTH*3+2];
-
-	    buf2[0] = '\0';
-	    // FIXME: Add RTCMV3 handling as well.
-#ifdef RTCM104V2_ENABLE
-	    if ((session->gpsdata.set & RTCM2_SET) != 0)
-		rtcm2_dump(&session->gpsdata.rtcm2,
-			  buf2+strlen(buf2),
-			  (sizeof(buf2)-strlen(buf2)));
-	    else {
-#endif /* RTCM104V2_ENABLE */
-#ifdef BINARY_ENABLE
-		gpsd_binary_dump(session, buf2, sizeof(buf2));
-#endif /* BINARY_ENABLE */
-#ifdef RTCM104V2_ENABLE
-	    }
-#endif /* RTCM104V2_ENABLE */
-	    if (buf2[0] != '\0') {
-		gpsd_report(LOG_IO, "<= GPS (binary) %s: %s",
-			    session->gpsdata.gps_device, buf2);
-		if (session->gpsdata.raw_hook)
-		    session->gpsdata.raw_hook(&session->gpsdata,
-					      buf2, strlen(buf2), 1);
-	    }
+	/*
+	 * Sanity check.  This catches a surprising number of port and
+	 * driver errors, including 32-vs.-64-bit problems.
+	 */
+	/*@+relaxtypes +longunsignedintegral@*/
+	if ((session->gpsdata.set & TIME_SET)!=0) {
+	    if (session->gpsdata.fix.time > time(NULL) + (60 * 60 * 24 * 365))
+		gpsd_report(LOG_ERROR,"date more than a year in the future!\n");
+	    else if (session->gpsdata.fix.time < 0)
+		gpsd_report(LOG_ERROR,"date is negative!\n");
 	}
-
-	if (session->gpsdata.fix.mode == MODE_3D)
-	    dgnss_report(session);
+	/*@-relaxtypes -longunsignedintegral@*/
 
 	return session->gpsdata.set;
     }
@@ -838,6 +736,6 @@ void gpsd_zero_satellites(/*@out@*/struct gps_data_t *out)
     (void)memset(out->elevation, 0, sizeof(out->elevation));
     (void)memset(out->azimuth,	 0, sizeof(out->azimuth));
     (void)memset(out->ss,	 0, sizeof(out->ss));
-    out->satellites = 0;
+    out->satellites_visible = 0;
 }
 
