@@ -1,4 +1,4 @@
-/* $Id: monitor_italk.c 5400 2009-03-06 20:16:48Z esr $ */
+/* $Id: monitor_italk.c 6566 2009-11-20 03:51:06Z esr $ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -12,6 +12,7 @@
 #include <assert.h>
 
 #include "gpsd_config.h"
+
 #ifdef HAVE_NCURSES_H
 #include <ncurses.h>
 #else
@@ -77,7 +78,7 @@ static bool italk_initialize(void)
 	/*@ +onlytrans @*/
 }
 
-static void display_itk_navfix(unsigned char *buf, size_t len){
+static void display_itk_navfix(unsigned char *buf, size_t len) {
 
 	unsigned int tow, tod, nsec, d, svlist;
 	unsigned short gps_week, flags, cflags, pflags, nsv;
@@ -90,23 +91,23 @@ static void display_itk_navfix(unsigned char *buf, size_t len){
 	if (len != 296)
 		return;
 
-	flags = getleuw(buf, 7 + 4);
-	cflags = getleuw(buf, 7 + 6);
-	pflags = getleuw(buf, 7 + 8);
+	flags = (ushort)getleuw(buf, 7 + 4);
+	cflags = (ushort)getleuw(buf, 7 + 6);
+	pflags = (ushort)getleuw(buf, 7 + 8);
 
 #define MAX(a,b) (((a) > (b)) ? (a) : (b))
-	nsv = MAX(getleuw(buf, 7 + 12), getleuw(buf, 7 + 14));
-	svlist = getleul(buf, 7 + 16) | getleul(buf, 7 + 24);
+	nsv = (ushort)MAX(getleuw(buf, 7 + 12), getleuw(buf, 7 + 14));
+	svlist = (ushort)getleul(buf, 7 + 16) | getleul(buf, 7 + 24);
 
-	hour = getleuw(buf, 7 + 66);
-	min = getleuw(buf, 7 + 68);
-	sec = getleuw(buf, 7 + 70);
-	nsec = getleul(buf, 7 + 72);
-	year = getleuw(buf, 7 + 76);
-	mon = getleuw(buf, 7 + 78);
-	day = getleuw(buf, 7 + 80);
+	hour = (ushort)getleuw(buf, 7 + 66);
+	min = (ushort)getleuw(buf, 7 + 68);
+	sec = (ushort)getleuw(buf, 7 + 70);
+	nsec = (ushort)getleul(buf, 7 + 72);
+	year = (ushort)getleuw(buf, 7 + 76);
+	mon = (ushort)getleuw(buf, 7 + 78);
+	day = (ushort)getleuw(buf, 7 + 80);
 	gps_week = (ushort)getlesw(buf, 7 + 82);
-	tow = getleul(buf, 7 + 84);
+	tow = (ushort)getleul(buf, 7 + 84);
 
 	epx = (double)(getlesl(buf, 7 + 96)/100.0);
 	epy = (double)(getlesl(buf, 7 + 100)/100.0);
@@ -172,8 +173,8 @@ static void display_itk_navfix(unsigned char *buf, size_t len){
 		char prn[4], satlist[38];
 		unsigned int i;
 		satlist[0] = '\0';
-		for(i = 0; i<32; i++){
-			if (svlist & (1<<i)){
+		for(i = 0; i<32; i++) {
+			if (svlist & (1<<i)) {
 				(void)snprintf(prn, 4, "%u ", i+1);
 				(void)strlcat(satlist, prn, 38);
 			}
@@ -198,7 +199,7 @@ static void display_itk_prnstatus(unsigned char *buf, size_t len)
 		unsigned short fl;
 		unsigned char ss, prn, el, az;
 
-		fl  = getleuw(buf, off);
+		fl  = (unsigned short)getleuw(buf, off);
 		ss  = (unsigned char)getleuw(buf, off+2)&0xff;
 		prn = (unsigned char)getleuw(buf, off+4)&0xff;
 		el  = (unsigned char)getlesw(buf, off+6)&0xff;
@@ -208,7 +209,7 @@ static void display_itk_prnstatus(unsigned char *buf, size_t len)
 			prn, az, el, ss, fl,
 			(fl & PRN_FLAG_USE_IN_NAV)? 'Y' : ' ');
 	}
-	for ( ; i < MAX_NR_VISIBLE_PRNS; i++){
+	for ( ; i < MAX_NR_VISIBLE_PRNS; i++) {
 	    (void)wmove(satwin, (int)i+2, 4);
 		(void)wprintw(satwin, "                      ");
 	}
@@ -224,7 +225,7 @@ static void italk_update(void)
 
 	buf = session.packet.outbuffer;
 	len = session.packet.outbuflen;
-	type = getub(buf, 4);
+	type = (unsigned char)getub(buf, 4);
 	switch (type) {
 		case ITALK_NAV_FIX:
 			display_itk_navfix(buf, len);
