@@ -1,4 +1,5 @@
-/* $Id: packet_names.h 4644 2007-12-28 03:13:44Z esr $ */
+/* $Id: packet_names.h 5771 2009-07-16 22:22:05Z ckuethe $ */
+/* edit packet_names.h to add new packet types. */
    "GROUND_STATE",	/* we don't know what packet type to expect */
 
    "COMMENT_BODY",	/* pound comment for a test load */
@@ -64,9 +65,11 @@
    "ZODIAC_RECOGNIZED",	/* found end of the Zodiac packet */
 #endif /* ZODIAC_ENABLE */
 
-#if defined(TNT_ENABLE) || defined(GARMINTXT_ENABLE)
-   "TNT_LEADER",          /* saw True North status leader '@' */
-                        /* Garmin Simple Text starts with @ leader */
+#if defined(TNT_ENABLE) || defined(GARMINTXT_ENABLE) || defined(ONCORE_ENABLE)
+   "AT1_LEADER",		/* saw True North status leader '@' */
+			/* Garmin Simple Text starts with @ leader */
+			/* Oncore starts with @ leader */
+   "GTXT_RECOGNIZED",     /* */
 #endif
 
 #ifdef EVERMORE_ENABLE
@@ -112,6 +115,26 @@
    "UBX_RECOGNIZED",      /* this is also UBX_CHECKSUM_B */
 #endif
 
+#ifdef SUPERSTAR2_ENABLE
+   "SUPERSTAR2_LEADER",	/* leading SOH */
+   "SUPERSTAR2_ID1",	/* message type */
+   "SUPERSTAR2_ID2",	/* message type xor 0xff */
+   "SUPERSTAR2_PAYLOAD",	/* length of the actual packet data */
+   "SUPERSTAR2_CKSUM1",
+   "SUPERSTAR2_CKSUM2",
+   "SUPERSTAR2_RECOGNIZED",
+#endif
+
+#ifdef ONCORE_ENABLE
+   "ONCORE_AT2",		/* second @ */
+   "ONCORE_ID1",		/* first character of command type */
+   "ONCORE_PAYLOAD",	/* payload eating */
+   "ONCORE_CHECKSUM",	/* checksum byte */
+   "ONCORE_CR",		/* closing CR */
+   "ONCORE_RECOGNIZED",	/* closing LF */
+#endif
+
+
 /*
  * Packet formats without checksums start here.  We list them last so
  * that if a format with a conflicting structure *and* a checksum can
@@ -126,10 +149,17 @@
    "GARMIN_RECOGNIZED",	/* found end of Garmin packet */
 #endif /* TSIP_ENABLE GARMIN_ENABLE */
 
-#ifdef RTCM104_ENABLE
-   "RTCM_SYNC_STATE",	/* we have sync lock */
-   "RTCM_SKIP_STATE",	/* we have sync lock, but this character is bad */
-   "RTCM_RECOGNIZED",	/* we have an RTCM packet */
-#endif /* RTCM104_ENABLE */
+#ifdef RTCM104V2_ENABLE
+   "RTCM2_SYNC_STATE",	/* we have sync lock */
+   "RTCM2_SKIP_STATE",	/* we have sync lock, but this character is bad */
+   "RTCM2_RECOGNIZED",	/* we have an RTCM packet */
+#endif /* RTCM104V2_ENABLE */
+
+#ifdef RTCM104V3_ENABLE
+   "RTCM3_LEADER_1",      /* constant leader byte found */
+   "RTCM3_LEADER_2",      /* second leader byte found (high 6 bits zero) */
+   "RTCM3_PAYLOAD",       /* gathering payload */
+   "RTCM3_RECOGNIZED",    /* RTCM3 packet recognized */
+#endif
 
 /* end of packet_names.h */
